/* This program generates a dynagraph data file.  Output goes to stdout.

   Run the program, redirect stdout to a file, then display the image
   in dynagraph with the command:
       dataplot("filename");

   The image showns a torus which has been split in two
   equal halves via a cut with a Mobius strip.  Well, not a Mobius strip
   really, but a similar strip with _two_ half-turns, which is not a
   one-sided surface anymore.

   See http://www.cs.berkeley.edu/~sequin/SCULPTS/KEIZO for a
   sculpture of the same in granite made by Keizo Ushio.

   -- RR, August 1999
*/

#include <stdio.h>
#include <math.h>

#define PI	3.14159265358979323846

int main(void)
{
	float r = 0.5;
	float R = 1.1;
	int twists = 2;

	float x, y, z;
	float a, s, t, v;
	int i, j;
	int m = 40;
	int n;

	printf("# Display this datafile in dynagraph by:\n");
	printf("#  dataplot(`filename`);\n");
	printf("\n");
	printf("TITLE 'http://www.cs.berkeley.edu/~sequin/SCULPTS/KEIZO'\n\n");

	/* the flat part */
	n = 5;
	printf("SURF %d %d\n", m, n);
	for (i=0; i<m; i++)  {
		t = i*2.0*PI/(m-1);		/* longitudinal angle */
		a = t*twists/2.0;		/* slant of the Mobius "ribs" */
		for (j=0; j<n; j++) {
			v = -r + j*(2*r)/(n-1);
			x = (R + v*cos(a))*cos(t);
			y = (R + v*cos(a))*sin(t);
			z = v*sin(a);
			printf("%g %g %g\n", x, y, z);
		}
	}

	/* the rounded part */
	n = 9;
	printf("SURF %d %d\n", m, n);
	for (i=0; i<m; i++)  {
		t = i*2.0*PI/(m-1);		/* longitudinal angle */
		a = t*twists/2.0;		/* slant of the Mobius "ribs" */
		for (j=0; j<n; j++) {
			s = j*PI/(n-1);
			x = (R + r*cos(a+s))*cos(t);
			y = (R + r*cos(a+s))*sin(t);
			z = r*sin(a+s);
			printf("%g %g %g\n", x, y, z);
		}
	}


	/* the flat part */
	n = 5;
	printf("SURF %d %d\n", m, n);
	for (i=0; i<m; i++)  {
		t = i*2.0*PI/(m-1);		/* longitudinal angle */
		a = t*twists/2.0;		/* slant of the Mobius "ribs" */
		for (j=0; j<n; j++) {
			v = -r + j*(2*r)/(n-1);
			y = (R + v*cos(a))*cos(t);
			y += (R+r)/2;
			z = (R + v*cos(a))*sin(t);
			x = v*sin(a);
			printf("%g %g %g\n", x, y, z);
		}
	}

	/* the rounded part */
	n = 9;
	printf("SURF %d %d\n", m, n);
	for (i=0; i<m; i++)  {
		t = i*2.0*PI/(m-1);		/* longitudinal angle */
		a = t*twists/2.0;		/* slant of the Mobius "ribs" */
		for (j=0; j<n; j++) {
			s = j*PI/(n-1);
			s += PI;
			y = (R + r*cos(a+s))*cos(t);
			y += (R+r)/2;
			z = (R + r*cos(a+s))*sin(t);
			x = r*sin(a+s);
			printf("%g %g %g\n", x, y, z);
		}
	}

	return 0;
}
