#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <readline/readline.h>
#include <forms.h>
#include <GL/gl.h>
#include "dynagraph.h"
#include "formdefs.h"

#include <tiffio.h>
#include <jpeglib.h>
#include <png.h>
#include <zlib.h>

extern char *timestamp;		/* defined in timestamp.c */
static FD_about *About = NULL;

/*  This routine is activated when the user closes the About form with the
    window manager's "Close" or "Delete" function.  By returning FL_OK, we
    are allowing the widow to be hidden, but xforms continues to run.
*/
static int atclose_cb(FL_FORM *form, void *data)
{
	return FL_OK;
}

static void make_about_form(void)
{
	int flversion, flrevision;
	char buf[1024];
	char *tiff_ver, *t;

	/* if the form is already on the screen, move it where the mouse is */
	if (About!=NULL) {
		fl_hide_form(About->about);
		fl_show_form(About->about, FL_PLACE_MOUSE, FL_FULLBORDER,
				ABOUT_FORM_NAME);
		return;
	}

	/* Get XForms's version */
	fl_library_version(&flversion, &flrevision);

	/* Tiff's TIFFGetVersion() function returns a multi-line string.
	   Here we extract the version number.
	*/
	tiff_ver = NULL;
	t = (char *)TIFFGetVersion();
	t = strstr(t, "Version ");
	if (t!=NULL) {
		t += 8;
		tiff_ver = copy_string(t);
		t = strchr(tiff_ver, '\n');
		if (t!=NULL)
			*t = '\0';
	}

	/* We hope that the buffer length does not exceed the allocated
	   size.   Last I checked, the buffer length was 488.
	*/
	sprintf(buf,
			"\n"
			"Dynagraph version %s (last modified %s)\n"
			"%s\n"
			"\n"
			"Dynagraph home:\n"
			"    http://www.math.umbc.edu/~rouben/dynagraph/\n"
			"\n"
			"This program has been compiled with the following libraries:\n"
			"    OpenGL:   version %s by %s\n"
			"    XForms:   version %d.%d\n"
			"    readline: version %s\n"
			"    TIFF:     version %s\n"
			"    jpeg:     version %d\n"
			"    PNG:      version %s\n"
			"    zlib:     version %s\n"
			"\n",
			
			DYNAGRAPH_VERSION, timestamp,
			COPYRIGHT,
			glGetString(GL_VERSION), glGetString(GL_VENDOR),
			flversion, flrevision,
			rl_library_version,
			tiff_ver!=NULL ? tiff_ver : "No version info available",
			JPEG_LIB_VERSION,
			PNG_LIBPNG_VER_STRING,
			ZLIB_VERSION
			);

/* 	printf("length of buf in %s = %d\n", __FILE__, strlen(buf)); */

	About = create_form_about();
	fl_set_form_atclose(About->about, atclose_cb, NULL);
	fl_set_object_label(About->text, buf);
	fl_fit_object_label(About->text, 0, 0);
	free(tiff_ver);
}

/* callback function for the "About" window's "OK" button */
void hide_about_form(FL_OBJECT *obj, long arg)
{
	fl_hide_form(obj->form);	/* same as fl_hide_form(About->about);*/
}

/* callback function for the main window's "About" button */
void show_about_form(FL_OBJECT *obj, long arg)
{
	if (About==NULL)
		make_about_form();

	/* Since the currently shown form may have been possibly relocated
	   on the screen.  We hide, then show it at the mouse. */
	if (fl_form_is_visible(About->about))
		fl_hide_form(About->about);

	fl_show_form(About->about,
			FL_PLACE_MOUSE, FL_FULLBORDER, ABOUT_FORM_NAME);
}
