#include <float.h>	/* for DBL_EPSILON */
#include <math.h>	/* for pow() */
#include <assert.h>
#include "dynagraph.h"
#include "gl2ps.h"
#include <GL/gl.h>

extern GLfloat	*fgColor;
extern gl2psContext *gl2ps;

static double get_first_tick(double a, double b, int minticks, double *stp)
{
	double r, r10, step, firsttick;
	int nticks;
	double eps = DBL_EPSILON;

	r = floor(log10(b-a));
	r10 = pow(10,r);

	step = r10;
	firsttick = step * floor((a+step-eps)/step);
	nticks = 1 + (int)floor((b-firsttick)/step);

	while (nticks<minticks) {
		step = step/2;
		firsttick = step * floor((a+step-eps)/step);
		nticks = 1 + (int)floor((b-firsttick)/step);

		if (nticks>=minticks)
			break;

		step = 2*step/5;
		firsttick = step * floor((a+step-eps)/step);
		nticks = 1 + (int)floor((b-firsttick)/step);
	}

	*stp = step;
	return firsttick;
}

static void print_labels(Plot3Dattribs *a)
{
	double t, t0, step;
	double ticklen, ticklenx, tickleny, ticklenz;
	double xmin, xmax, ymin, ymax, zmin, zmax;
	char label[32];
	double A=8, B=12;
	GLuint axesfont_glbase, labelfont_glbase;
	Fontdesc *f;
	char *pslabelfontname, *psaxesfontname;
	int pslabelfontsize, psaxesfontsize;

	xmin = a->xmin;
	ymin = a->ymin;
	zmin = a->zmin;
	xmax = a->xmax;
	ymax = a->ymax;
	zmax = a->zmax;

	/* the half-length of the tick lines */
	ticklen = a->radius/80;
	if ((a->scaling&~locked)==UNCONSTRAINED) {
		ticklenx = ticklen/a->xscale;
		tickleny = ticklen/a->yscale;
		ticklenz = ticklen/a->zscale;
	}
	else
		ticklenx = tickleny = ticklenz = ticklen;

	f = getfont(a->axesfont&~locked);
	assert(f!=NULL);
	axesfont_glbase = f->glbase;

	f = getfont(a->labelfont&~locked);
	assert(f!=NULL);
	labelfont_glbase = f->glbase;

	if (gl2ps) {
		get_PostscriptFont(a->axesfont&~locked,
				&psaxesfontname, &psaxesfontsize);
		get_PostscriptFont(a->labelfont&~locked,
				&pslabelfontname, &pslabelfontsize);
	}

	/********************** The X AXIS **********************/

	/* put tick lines */
	if (xmax-xmin < 1e-3)
		goto no_xticks;

	if (a->xticks==0)
		goto no_xticks;

	glBegin(GL_LINES);
	t0 = t = get_first_tick(xmin, xmax, a->xticks, &step);
	while (t<xmax+step/2) {
		glVertex3f(t, ymin - tickleny, zmin - ticklenz);
		glVertex3f(t, ymin + tickleny, zmin + ticklenz);
		t += step;
	}
	glEnd();

	/* put numbers */
	t = t0;
	while (t<xmax+step/2) {
		glRasterPos3f(t, ymin - A*tickleny, zmin - A*ticklenz);
		sprintf(label, "% g", t);
		if (gl2ps) gl2psText(label, psaxesfontname, psaxesfontsize);
		print_string(label, axesfont_glbase);
		t += step;
	}

no_xticks:

	if (a->coords==CARTESIAN) {	/* put axis label */
		glRasterPos3f((xmin+xmax)/2, ymin-B*tickleny, zmin-B*ticklenz);
		if (gl2ps)
			gl2psText(a->xlabel, pslabelfontname, pslabelfontsize);
		print_string(a->xlabel, labelfont_glbase);
	}

	/********************** The Y AXIS **********************/

	/* put tick lines */
	if (ymax-ymin < 1e-3)
		goto no_yticks;

	if (a->yticks==0)
		goto no_yticks;

	glBegin(GL_LINES);
	t0 = t = get_first_tick(ymin, ymax, a->yticks, &step);
	while (t<ymax+step/2) {
		glVertex3f(xmin - ticklenx, t, zmin - ticklenz);
		glVertex3f(xmin + ticklenx, t, zmin + ticklenz);
		t += step;
	}
	glEnd();

	/* put numbers */
	t = t0;
	while (t<ymax+step/2) {
		glRasterPos3f(xmin - A*ticklenx, t, zmin - A*ticklenz);
		sprintf(label, "% g", t);
		if (gl2ps) gl2psText(label, psaxesfontname, psaxesfontsize);
		print_string(label, axesfont_glbase);
		t += step;
	}

no_yticks:

	if (a->coords==CARTESIAN) {	/* put axis label */
		glRasterPos3f(xmin-B*ticklenx, (ymin+ymax)/2, zmin-B*ticklenz);
		if (gl2ps)
			gl2psText(a->ylabel, pslabelfontname, pslabelfontsize);
		print_string(a->ylabel, labelfont_glbase);
	}

	/********************** The Z AXIS **********************/

	/* put tick lines */
	if (zmax-zmin < 1e-3)
		goto no_zticks;

	if (a->zticks==0)
		goto no_zticks;

	glBegin(GL_LINES);
	t0 = t = get_first_tick(zmin, zmax, a->zticks, &step);
	while (t<zmax+step/2) {
		glVertex3f(xmin - ticklenx, ymin - tickleny, t);
		glVertex3f(xmin + ticklenx, ymin + tickleny, t);
		t += step;
	}
	glEnd();

	/* put numbers */
	t = t0;
	while (t<zmax+step/2) {
		glRasterPos3f(xmin - A*ticklenx, ymin - A*tickleny, t);
		sprintf(label, "% g", t);
		if (gl2ps) gl2psText(label, psaxesfontname, psaxesfontsize);
		print_string(label, axesfont_glbase);
		t += step;
	}

no_zticks:

	if (a->coords==CARTESIAN) {	/* put axis label */
		glRasterPos3f(xmin-B*ticklenx, ymin-B*tickleny, (zmin+zmax)/2);
		if (gl2ps)
			gl2psText(a->zlabel, pslabelfontname, pslabelfontsize);
		print_string(a->zlabel, labelfont_glbase);
	}
}

void compile_boundingbox(Plot3Dattribs *a)
{
	generate_font(a->axesfont&~locked);
	generate_font(a->labelfont&~locked);

	/* the display list a->idx_bbox has been assigned in compile_objects()*/
	if (!gl2ps) glNewList(a->idx_bbox, GL_COMPILE);

	glPushAttrib(GL_LIGHTING_BIT);

	glDisable(GL_LIGHTING);
	glColor4fv(fgColor);


	glPushMatrix();

	/*
       a ----- b	idea for drawing cube from:
      /|      /|	Scott McMillan (mcmillan@cambridge.com)
     / |     / |
    d ----- c  |        it traces out the edges of the cube
    |  |    |  |        in 15 consecutive line segments,
    |  e ---|- f        thus 3 segments are doubly traced
    | /     | /
    |/      |/
    h-------g
	*/

	glLineWidth(2);
	gl2psLineWidth(2);
	glBegin(GL_LINE_STRIP);
	glVertex3f(a->xmin, a->ymin, a->zmin);	/* e */
	glVertex3f(a->xmin, a->ymin, a->zmax);	/* a */
	glVertex3f(a->xmin, a->ymax, a->zmax);	/* b */
	glVertex3f(a->xmax, a->ymax, a->zmax);	/* c */
	glVertex3f(a->xmax, a->ymin, a->zmax);	/* d */
	glVertex3f(a->xmax, a->ymin, a->zmin);	/* h */
	glVertex3f(a->xmax, a->ymax, a->zmin);	/* g */
	glVertex3f(a->xmin, a->ymax, a->zmin);	/* f */
	glVertex3f(a->xmin, a->ymin, a->zmin);	/* e */
	glVertex3f(a->xmax, a->ymin, a->zmin);	/* h */
	glVertex3f(a->xmax, a->ymin, a->zmax);	/* d */
	glVertex3f(a->xmin, a->ymin, a->zmax);	/* a */
	glVertex3f(a->xmin, a->ymax, a->zmax);	/* b */
	glVertex3f(a->xmin, a->ymax, a->zmin);	/* f */
	glVertex3f(a->xmax, a->ymax, a->zmin);	/* g */
	glVertex3f(a->xmax, a->ymax, a->zmax);	/* c */
	glEnd();

	print_labels(a);

	glPopMatrix();
	glPopAttrib();	/* GL_LIGHTING_BIT */
	if (!gl2ps) glEndList();
}
