#include <math.h>	/* for sqrt() */
#include "dynagraph.h"
#include "parse.h"

void compute_bounds(Plot3D *p)
{
	int i, j;

	for (i=0; i<p->attr->m; i++)
		for (j=0; j<p->attr->n; j++) {
			if (IS_SINGULAR(p->mesh[i][j]))
				continue;
			p->xmin = MIN(p->mesh[i][j].x, p->xmin);
			p->xmax = MAX(p->mesh[i][j].x, p->xmax);
			p->ymin = MIN(p->mesh[i][j].y, p->ymin);
			p->ymax = MAX(p->mesh[i][j].y, p->ymax);
			p->zmin = MIN(p->mesh[i][j].z, p->zmin);
			p->zmax = MAX(p->mesh[i][j].z, p->zmax);
		}
}

void compute_curvebounds(Plot3D *p)
{
	int i;

	for (i=0; i<p->attr->n; i++) {
		if (IS_SINGULAR(p->curvepoints[i]))
			continue;
		p->xmin = MIN(p->curvepoints[i].x, p->xmin);
		p->xmax = MAX(p->curvepoints[i].x, p->xmax);
		p->ymin = MIN(p->curvepoints[i].y, p->ymin);
		p->ymax = MAX(p->curvepoints[i].y, p->ymax);
		p->zmin = MIN(p->curvepoints[i].z, p->zmin);
		p->zmax = MAX(p->curvepoints[i].z, p->zmax);
	}
}

void compute_centroid(Plot3Dattribs *a)
{
	double f;

	a->radius =
		(a->xmax - a->xmin)*(a->xmax - a->xmin) +
		(a->ymax - a->ymin)*(a->ymax - a->ymin) +
		(a->zmax - a->zmin)*(a->zmax - a->zmin) ;
	a->radius = sqrt(a->radius) / 2;

	a->cx = (a->xmax + a->xmin)/2;
	a->cy = (a->ymax + a->ymin)/2;
	a->cz = (a->zmax + a->zmin)/2;

	/* To display in the unconstrained scaling mode, we scale the x, y, and
	   z dimensions of the bounding box of the scene to make them all equal.
	   The scaling factors are chosen such that the lenght of the diameter
	   of the bounding box remains unchanged after scaling.
	 
	   This segment requires some work.  Precautions should be taken to 
	   avoid division by zero for flat objects.
	*/
	f = a->radius / (sqrt(3)/2);
	a->xscale = f/(a->xmax - a->xmin);
	a->yscale = f/(a->ymax - a->ymin);
	a->zscale = f/(a->zmax - a->zmin);
}
