#ifndef bspH
#define bspH

#undef NDEBUG
#include <assert.h>

/* uncomment the line below to see a lot of debugging messages */
/* #define DEBUG */

#ifdef DEBUG
#define PRINTF(x)	printf x
#else
#define PRINTF(x)
#endif

typedef enum {
	bspCoplanar,
	bspInFront,
	bspInBack,
	bspSplit2,
	bspSplit3
} bspCompare;

typedef enum {
	bspTextObj,
	bspPointObj,
	bspLineObj,
	bspTriangleObj
} bspObjType;

typedef struct {
	char *str;
	char *fontname;
	float fontsize;
} Text;

typedef double bspPoint[3];	/* coords of points or components of vectors */
typedef double bspPlane[4];	/* Ax + By + Cz = D */
typedef float  bspColor[4];	/* RGBA components */

typedef struct bspObj {
	bspObjType type;
	short nverts;		/* number of vertices */
#ifdef DEBUG
	char *name;		/* name, for debugging */
#endif /* DEBUG */
	bspPoint *v;		/* vertices */
	bspColor *rgba;		/* colors */
	double depth;
	float linewidth;	/* for a bspLineObj */
	float pointsize;	/* for a bspPointObj */
	Text *text;		/* for a bspTextObj */
	struct bspObj *next;
} bspObj;

typedef struct bspNode {
	bspObj *obj, *last;
	bspPlane plane;		/* the partitioning plane */
	struct bspNode *left;
	struct bspNode *right;
} bspNode;

void bspObjAdd(bspNode *node, bspObj *obj);
void bspProduceTree(bspNode *rootnode);
bspObj *bspMakeObject(bspObjType type, short nverts);
double bspDotprod(const bspPoint a, const bspPoint b);
bspNode *bspMakeNode(void);
void bspSimpleSort(bspNode *node);
void bspFreeTree(bspNode *node);

#endif /* bspH */
