#include <forms.h>
#include <GL/gl.h>
#include "formdefs.h"
#include "dynagraph.h"

extern FD_form		*Form;
extern GLfloat		light0_diffuse[];
extern GLfloat		light0_ambient[];
extern GLfloat		light0_specular[];
double zoom_scale = 1.0;

void autorot_button_cb(FL_OBJECT *obj, long arg)
{
	return;
}

void toggle_lights(FL_OBJECT *obj, long arg)
{
	if (fl_get_button(obj))
		glEnable(GL_LIGHTING);
	else
		glDisable(GL_LIGHTING);

	if (arg==0)
		display();
}

void toggle_hilights(FL_OBJECT *obj, long arg)
{
	GLfloat	zerovect4[] = { 0, 0, 0, ALPHA };

	if (fl_get_button(obj))
		glLightfv(GL_LIGHT0, GL_SPECULAR, light0_specular);
	else
		glLightfv(GL_LIGHT0, GL_SPECULAR, zerovect4);

	if (arg==0)
		display();
}

/* Callback for the frames/sec or secs/frame radio buttons */
void invert_speed_data(FL_OBJECT *obj, long arg)
{
	char buf[32];
	double val;
	
	/* retrieve and invert the currently displayed value */
	val = Form->framespeed_box->u_ldata / 1.0e6;
	val = 1.0/val;

	/* show the inverted value */
	fl_get_button(Form->framecounterbutton)
		? sprintf(buf, "%.1f", val)
		: sprintf(buf, "%.3f", val);
	fl_set_object_label(Form->framespeed_box, buf);

	/* save the displayed value for future use */
	Form->framespeed_box->u_ldata = (long)(1.0e6 * val);
}


/* This callback is activated when the zoom dial returns.  It is also called
   by reset_zoom() and by set_view_menu_cb().  Normally, the routine is
   called with arg=0, in which case it scales the object relative to its
   current size to bring it up to a size read from the dial setting.
   Some other routine, such as routines set_view_menu_cb(), however, call
   this routine with arg=1, in which case the scaling is not relative, but
   absolute.
*/
void zoomdial_cb(FL_OBJECT *obj, long arg)
{
	double a, z, scale;
	const float m = MINZOOM;
	const float M = MAXZOOM;
	FL_OBJECT *glCanvas = Form->glCanvas;
	Plot3D *p = glCanvas->u_vdata;

	if (arg==1)
		scale = zoom_scale;
	else {
		/* get dial value (between 0.0 and 1.0) */
		z = fl_get_dial_value(obj);

		/* The following expression defines a(z) as the quotient of
		   two linear functions in z, (i.e., the graph of a(z) is a
		   hyperbola.)  The coefficients are set up so that
		   a(0)=m, a(1/2)=1, a(1)=M. */
		a = ( (2*M*m-M-m)*z - (M-1)*m ) / ( (M+m-2)*z - (M-1) );

		/* The value of 'a' computed above is the absolute
		   value of the zoom factor, measured relative to the object's
		   natural size.  The glScale below scales the object relative
		   to its current size, which may not be the original size.
		   Here we adjust for this and save the current zoom setting
		   for the next round.
		*/
		scale = a/zoom_scale;
		zoom_scale = a;
	}

	glScalef(scale,scale,scale);
	compile_scene(p);
	
	display();
}

void reset_zoom(FL_OBJECT *obj, long arg)
{
	fl_set_dial_value(Form->zoomdial, 0.5001);
	zoomdial_cb(Form->zoomdial, 0);
}

void brightnessdial_cb(FL_OBJECT *obj, long arg)
{
	float m, M, x, a, A;

	/* read dial value (between 0.0 and 1.0) */
	x = fl_get_dial_value(obj);

	/* a(x) is a hyperbola such that: a(0)=m, a(1/2)=A, a(1)=M. Thus
	   it maps the range (0,1) to (m,M) and at 1/2 coinsides with
	   the default ambient intensity.  */

	m = LIGHT0_AMBIENT_MIN;
	M = LIGHT0_AMBIENT_MAX;
	A = LIGHT0_AMBIENT_DEFAULT;
	a = ((A*(M+m)-2*m*M)*x + m*(M-A)) / ((2*A-M-m)*x + (M-A));
	light0_ambient[0] = light0_ambient[1] = light0_ambient[2] = a;
	glLightfv(GL_LIGHT0, GL_AMBIENT, light0_ambient);

	m = LIGHT0_DIFFUSE_MIN;
	M = LIGHT0_DIFFUSE_MAX;
	A = LIGHT0_DIFFUSE_DEFAULT;
	a = ((A*(M+m)-2*m*M)*x + m*(M-A)) / ((2*A-M-m)*x + (M-A));
	light0_diffuse[0] = light0_diffuse[1] = light0_diffuse[2] = a;
	glLightfv(GL_LIGHT0, GL_DIFFUSE, light0_diffuse);

	display();
}

void reset_brightness(FL_OBJECT *obj, long arg)
{
	fl_set_dial_value(Form->brightnessdial, 0.5001);
	brightnessdial_cb(Form->brightnessdial, 0);
}

void toggle_box_cb(FL_OBJECT *obj, long arg)
{
	FL_OBJECT *glCanvas = Form->glCanvas;
	Plot3D *p = glCanvas->u_vdata;

	if (fl_get_button(obj))
		p->attr->axes = BOXED;
	else 
		p->attr->axes = NOAXES;

	compile_scene(p);
	display();
}

void toggle_constrained_scaling(FL_OBJECT *obj, long arg)
{
	FL_OBJECT *glCanvas = Form->glCanvas;

	Plot3D *p = glCanvas->u_vdata;

	if (fl_get_button(obj))
		p->attr->scaling = CONSTRAINED;
	else 
		p->attr->scaling = UNCONSTRAINED;

	compile_scene(p);
	display();
}
