#include <GL/gl.h>
#include "dynagraph.h"

#define isordered(a,b,c) (((a)<=(b) && (b)<=(c)) || ((a)>=(b) && (b)>=(c)))

/* Given two points p and q, and value elev, determines a point m on the
   line segment pq the z coordinate of which equals elev.  If such a point
   exists, returns 0, else returns -1.
*/
static int get_mid_coords(Point p, Point q, double elev, Point *m)
{
	double a, b;

	if (IS_SINGULAR(p) || IS_SINGULAR(q))
		return -1;

	if (!isordered(p.z, elev, q.z))
		return -1;

	if (p.z==q.z)
		a = 0.5;
	else
		a = (q.z-elev)/(q.z-p.z);

	b = 1 - a;
	m->x = a*p.x + b*q.x;
	m->y = a*p.y + b*q.y;
	m->z = a*p.z + b*q.z;

	return 0;
}

static void construct_segment(Plot3D *plot, Point p, Point q, GLfloat *color)
{
	if (color!=NULL)
		glColor4fv(color);

	glBegin(GL_LINE_STRIP);
	if (color==NULL)
		set_vertex_color(plot, p.x, p.y, p.z);
	glVertex3f(p.x, p.y, p.z);

	if (color==NULL)
		set_vertex_color(plot, q.x, q.y, q.z);
	glVertex3f(q.x, q.y, q.z);

	glEnd();
}	

static void construct_contour(Plot3D *p, double elev, GLfloat *color)
{
	int i, j, k, l;
	Point p1, q1, m1, p2, q2, m2;

	for (i=0; i<p->attr->m-1; i++)
		for (j=0; j<p->attr->n-1; j++)
			for (k=0; k<3; k++) {
				switch (k) {
					case 0:	p1 = p->mesh[i][j];
						q1 = p->mesh[i][j+1];
						break;
					case 1:	p1 = p->mesh[i][j+1];
						q1 = p->mesh[i+1][j+1];
						break;
					case 2:	p1 = p->mesh[i+1][j+1];
						q1 = p->mesh[i+1][j];
						break;
				}
				if (get_mid_coords(p1, q1, elev, &m1)==-1)
					continue;

				for (l=k+1; l<4; l++) {
					switch (l) {
						case 1:	p2 = p->mesh[i][j+1];
							q2 = p->mesh[i+1][j+1];
							break;
						case 2:	p2 = p->mesh[i+1][j+1];
							q2 = p->mesh[i+1][j];
							break;
						case 3:	p2 = p->mesh[i+1][j];
							q2 = p->mesh[i][j];
							break;
					}
					if (get_mid_coords(p2, q2, elev, &m2)==-1)
						continue;

					construct_segment(p, m1, m2, color);
				}
			}
}

void construct_contours(Plot3D *p, GLfloat *color)
{
	int ncontours;
	double elev;
	double zmin, zmax;
	int i;

	if (color!=NULL)
		glColor4fv(color);

	zmin = p->attr->z_viewmin==NULL ? p->attr->zmin : p->attr->zviewmin;
	zmax = p->attr->z_viewmax==NULL ? p->attr->zmax : p->attr->zviewmax;
	ncontours = p->attr->contours;

	for (i=0; i<ncontours; i++) {
		elev = zmin + i*(zmax-zmin)/(ncontours-1);
		construct_contour(p, elev, color);
	}
}
