#include <stdio.h>
#include <stdlib.h>
#include <string.h>	/* for strlen() */
#include <unistd.h>	/* for fork(), dup(), pipe(), etc. */
#include <math.h>	/* for sqrt() */
#include <signal.h>	/* for signal handlers */
#include <sys/types.h>	/* for signal handlers */
#include <sys/time.h>	/* for struct timeval */
#include <ctype.h>	/* for isspace() */
#include <readline/readline.h>
#include <readline/history.h>
#include <forms.h>
#include <GL/gl.h>
#include "dynagraph.h"
#include "parse.h"
#include "formdefs.h"

FD_form		*Form = NULL;
Plot3D		*dg_plots = NULL;
Plot3Dattribs	*dg_attribs = NULL;
char		*dg_dispargs = NULL;
extern Fontdesc	*fonttab;
extern char	*timestamp;		/* defined in timestamp.c */
static void	readline_cb(char *line);
char		*prognam;
int		restart;
GLfloat		whiteColor[4] = { 1.0, 1.0, 1.0, ALPHA };
GLfloat		blackColor[4] = { 0.0, 0.0, 0.0, ALPHA };
GLfloat		*bgColor = NULL;
GLfloat		*fgColor = NULL;

/* strips trailing whitespace in a string */
static void trim_tail(char *s)
{
	char *p = s;

	while (*p!='\0')
		p++;

	p--;

	while (p>=s && isspace((unsigned char)*p))
		*p-- = '\0';

}

void show_form(void)
{
	if (Form!=NULL)
		return;		/* form already shown */

	create_form();
	bgColor = whiteColor;
	fgColor = blackColor;
	fl_set_form_position(Form->form, 0, 0);
	fl_show_form(Form->form,
			FL_PLACE_POSITION, FL_FULLBORDER, MAIN_FORM_NAME);
	initialize_glCanvas();
	fonttab = NULL;
	generate_default_fonts();
	Form->glCanvas->u_vdata = NULL;
}

void create_displaylists(Plot3D *p)
{
	/* set scaling according to the scaling button state */
	if (!(p->attr->scaling&locked)) {
		if (Form!=NULL && fl_get_button(Form->constrained_scaling_button)==0)
			p->attr->scaling = UNCONSTRAINED;
		else
			p->attr->scaling = CONSTRAINED;
	}

	/* set axes status according to the scaling button state */
	if (!(p->attr->axes&locked)) {
		if (Form!=NULL && fl_get_button(Form->toggle_box_button)==1)
			p->attr->axes = BOXED;
		else
			p->attr->axes = NOAXES;
	}

	/* Is xlabel is null, so are the others, so we don't check all three */
	if (p->type==FUNCTION_PLOT && p->attr->coords==CARTESIAN
		 	&& p->attr->xlabel==NULL) {
		p->attr->xlabel = copy_string(p->attr->xrange->id);
		p->attr->ylabel = copy_string(p->attr->yrange->id);
		p->attr->zlabel = copy_string("");
	}

	compile_scene(p);
}

void read_from_string(char *line);	/* TERMPORARY */

/* call yyparse() for symbolic parsing of input */
int parse_line(char *line)
{
	enum parse_return ret;

	restart=1;
	read_from_string(line);
	while ((ret=yyparse())!=PP_EOI) {
		switch (ret) {
			case PP_ERROR:
				break;
			case PP_PLOT:		/* plot3d(...); */
			case PP_SPACECURVE:	/* spacecurve(...) */
			case PP_TUBEPLOT:	/* tubeplot(...) */
			case PP_TEXTPLOT:	/* textplot(...) */
			case PP_POLYGONPLOT:	/* polygonplot(...) */
				if (evaluate_plots(dg_plots)!=0) {
					show_form();
					create_displaylists(dg_plots);
					install_plot(dg_plots);
					/* show obj according to zoom setting */
					zoomdial_cb(Form->zoomdial, 1);
					/* don't free graph; it's now
					   attached to glCanvas->u_vdata */
					dg_plots = NULL;
					dg_attribs = NULL;
				}
				break;
			case PP_PLOT_SAVE:
			case PP_PLOT_SAVE_SHOW:
				if (evaluate_plots(dg_plots)!=0) {
					putplot(dg_plots);
					if (ret==PP_PLOT_SAVE_SHOW)
						show_sym(dg_attribs->id);
					/* don't free graph; it's now
					   stored in plottab */
					dg_plots = NULL;
					dg_attribs = NULL;
				}
				break;
			case PP_DISPLAY:
				dg_plots = make_merged_plot(dg_dispargs, dg_attribs);
				if (dg_plots!=NULL) {
					show_form();
					create_displaylists(dg_plots);
					install_plot(dg_plots);
					/* show obj according to zoom setting */
					zoomdial_cb(Form->zoomdial, 1);
				}
				/* don't free dg_plots; it's now
				   attached to glCanvas->u_vdata  */
				free_plot3d_attribs_structure(dg_attribs);
				testfree(dg_dispargs);
				dg_plots = NULL;
				dg_attribs = NULL;
				dg_dispargs = NULL;

				break;
			case PP_DISP_SAVE:
			case PP_DISP_SAVE_SHOW:
				dg_plots = make_merged_plot(dg_dispargs, dg_attribs);
				dg_plots->attr->id = copy_string(dg_attribs->id);
				putplot(dg_plots);
				if (ret==PP_DISP_SAVE_SHOW)
					show_sym(dg_plots->attr->id);
				/* don't free graph; it's now
				   stored on plottab */
				free_plot3d_attribs_structure(dg_attribs);
				testfree(dg_dispargs);
				dg_plots = NULL;
				dg_attribs = NULL;
				break;
			case PP_DATAPLOT:
				dg_plots = make_data_plot(dg_dispargs, dg_attribs);
				if (dg_plots==NULL)
					free_plot3d_attribs_structure(dg_attribs);
				else {
					show_form();
					create_displaylists(dg_plots);
					install_plot(dg_plots);
					/* show obj according to zoom setting */
					zoomdial_cb(Form->zoomdial, 1);
				}
				testfree(dg_dispargs);
				dg_attribs = NULL;
				dg_dispargs = NULL;

				break;
			case PP_DATAPLOT_SAVE:
			case PP_DATAPLOT_SAVE_SHOW:
				dg_plots = make_data_plot(dg_dispargs, dg_attribs);
				if (dg_plots==NULL)
					free_plot3d_attribs_structure(dg_attribs);
				else {
					putplot(dg_plots);
					if (ret==PP_DATAPLOT_SAVE_SHOW)
						show_sym(dg_attribs->id);
				}
				testfree(dg_dispargs);
				dg_attribs = NULL;
				dg_dispargs = NULL;

				break;
			case PP_ASGN:		/* r := a + b; */
			case PP_SHOW:		/* p; # p an expr or a plot */
			case PP_READ:		/* read filename; */
			case PP_SAVEGRAPH:	/* savegraph(...); */
			case PP_DUMP:		/* dump; */
			case PP_RESTART:	/* restart; */
				break;
			default:
				printf("SHOULD NOT BE HERE: line %d, file %s\n",
						__LINE__, __FILE__);
				break;
		}
		dg_dispargs = NULL;
		dg_plots = NULL;
		dg_attribs = NULL;
	}

	return 1;	/* TEMPORARY */
}

void call_exit(void)
{
	putchar('\n');
	exit(0);
}

/* does a preliminary processing of the line before handing it
   to the parser */
static void process_line(char *line)
{
	static char savebuf[1024];	/* TEMPORARY */
	char *p;
	int len;

	p = line;

	trim_tail(p);
	for (p=line; isspace((unsigned char)*p); p++)
		;

	if (p[0]=='\0')		/* empty line */
		return;

	if (strncmp(p,"quit",4)==0 || strncmp(p, "exit", 4)==0)
		call_exit();

	if (strncmp(p, "help", 4)==0 || strncmp(p, "?", 1)==0) {
		show_help(p);
		add_history(p);
		return;
	}

	if (p[0]=='!') {	/* shell escape */
		system(p+1);
		add_history(p);
		return;
	}

	/* concatenate multiple lines into one single line */
	strcat(savebuf, p);

	len = strlen(p);
	if (p[len-1]!=';' && p[len-1]!=':') {
		strcat(savebuf, " ");		/* add a spacer */
		rl_callback_handler_install(PROMPT2, readline_cb);
		return;
	}

	if (parse_line(savebuf))
		add_history(savebuf);

	rl_callback_handler_install(PROMPT1, readline_cb);
	savebuf[0] = '\0';
}

/* this is called when rl_callback_read_char() detects an end-of-line */
static void readline_cb(char *line)
{
	if (line==NULL)
		call_exit();	/* EOF reached */

	process_line(line);
}

void read_from_stdin(void)
{
	fd_set rfds;
	struct timeval tv;

	do {
		rl_callback_read_char();
		FD_ZERO(&rfds);
		FD_SET(0, &rfds);
		tv.tv_sec = 0;
		tv.tv_usec = 0;
	} while (select(1, &rfds, NULL, NULL, &tv));
}


/* This idle callback, the program's main execution loop, is on at all times.
   It does two things in each call:
   a) reads one character from stdin, if possible
   b) rotates the graph on forms with the autorot button on
*/
int idle_callback(XEvent *ev, void *data)
{
	fd_set rfds;
	struct timeval tv;
	static short int busy=0;

	if (!busy) {
		FD_ZERO(&rfds);
		FD_SET(0, &rfds);
		tv.tv_sec = 0;
		tv.tv_usec = 0;
		if (select(1, &rfds, NULL, NULL, &tv)) {
			busy=1;
			read_from_stdin();
			busy=0;
		}
	}

	/* if form is shown and autorot button is on, then rotate */
	if (Form!=NULL && Form->form->visible
			&& fl_get_button(Form->autorot_button))
		autorotate();

	return 0;	/* return value is ignored by xforms */
}

static void sig_handler(int sig, void *data)
{	
/* 	printf("*********** received signal %d *******\n", sig); */
}

int main(int argc, char *argv[])
{
	prognam = argv[0];

	if (fl_initialize(&argc, argv, APPCLASS, NULL, 0)==NULL)
		exit(1); /* diagnostic message will come from libform */

	if (fl_library_version(NULL, NULL) != FL_INCLUDE_VERSION) {
			printf("%s error: mismatched XForms library libforms.* and header file forms.h\n", prognam);
			exit(1);
	}

	printf("   Dynagraph version %s, (last modified %s)\n",
			DYNAGRAPH_VERSION, timestamp);
	printf("   %s\n", COPYRIGHT);

	fl_add_signal_callback(SIGTTIN, sig_handler, NULL);
	fl_add_signal_callback(SIGTTOU, sig_handler, NULL);

	rl_callback_handler_install(PROMPT1, readline_cb);
	fl_set_idle_callback(idle_callback, NULL);
	fl_do_forms();

	return 0;
}
