#ifndef DG_DYNAGRAPH_H
#define DG_DYNAGRAPH_H

#include <stdio.h>
#include <math.h>	/* for isfinite(), in C99 */
#include "forms.h"
#include <GL/gl.h>
#include "formdefs.h"

#define BROKEN_GL_NORMALIZE

#define DYNAGRAPH_VERSION	"1.3f"
#define COPYRIGHT	"Copyright (c) 1997-2003, Rouben Rostamian <rostamian@umbc.edu>"

#define APPCLASS	"Dynagraph"
#define MAIN_FORM_NAME	"Dynagraph Window"
#define ABOUT_FORM_NAME	"About Dynagraph"

#ifdef DEBUG
#define PRINTF(x)        printf x
#else
#define PRINTF(x)
#endif

#ifndef MAX
#define MAX(a,b)        ((a)>(b)?(a):(b))
#endif

#ifndef MIN
#define MIN(a,b)        ((a)<(b)?(a):(b))
#endif

#ifdef C99
#define IS_SINGULAR(p) (!(isfinite(p.x) && isfinite(p.y) && isfinite(p.z)))
#else
#define IS_SINGULAR(p) (!(finite(p.x) && finite(p.y) && finite(p.z)))
#endif

#ifndef M_PI
#define M_PI		3.14159265358979323846
#endif /* M_PI */

#define ALPHA		1.0  /* alpha-blending component, which is not used */

#define testfree(p)     if (p) { free(p); p=NULL; }

#define locked		(1<<30)	/* this bit should not overalap with
			      	   any of the ones defined below */
#define FUNCTION_PLOT	(1<<0)	/* plot type (function) */
#define PARAM_PLOT	(1<<1)	/* plot type (parametric) */
#define CURVE_PLOT	(1<<2)	/* plot type (spacecuve) */
#define TUBE_PLOT	(1<<3)	/* plot type (tubeplot) */
#define TEXT_PLOT	(1<<4)	/* plot type (textplot) */
#define POLYGON_PLOT	(1<<5)	/* plot type (polygonplot) */

#define POINT		(1<<0)	/* plot style */
#define HIDDEN		(1<<1)	/* plot style */
#define PATCH		(1<<2)	/* plot style */
#define WIREFRAME	(1<<3)	/* plot style */
#define CONTOUR		(1<<4)	/* plot style */
#define PATCHNOGRID	(1<<5)	/* plot style */
#define PATCHCONTOUR	(1<<6)	/* plot style */

#define XY_GRIDLINES	(1<<0)	/* grid style */
#define X_GRIDLINES	(1<<1)	/* grid style */
#define Y_GRIDLINES	(1<<2)	/* grid style */
#define TRIANGULAR_GRIDLINES	(1<<3)	/* grid style */

#define BOXED		(1<<0)	/* axes style */
#define NORMAL		(1<<1)	/* axes style */
#define FRAME		(1<<2)	/* axes style */
#define NOAXES		(1<<3)	/* axes style */

#define CONSTRAINED	(1<<0)	/* scaling style */
#define UNCONSTRAINED	(1<<1)	/* scaling style */

#define CARTESIAN	(1<<0)	/* coords option */
#define CYLINDRICAL	(1<<1)	/* coords option */
#define Z_CYLINDRICAL	(1<<2)	/* coords option */
#define SPHERICAL	(1<<3)	/* coords option */

#define XYZSHADING	(1<<0)	/* shading style */
#define XYSHADING	(1<<1)	/* shading style */
#define ZSHADING	(1<<2)	/* shading style */
#define ZGRAYSCALE	(1<<3)	/* shading style */
#define ZHUE		(1<<4)	/* shading style */
#define NOSHADING	(1<<5)	/* shading style */

			/* bits 0-7 reserved for font size */
#define TIMES		(1<<8)	/* font family */
#define COURIER		(1<<9)	/* font family */
#define HELVETICA	(1<<10)	/* font family */
#define SYMBOL		(1<<11)	/* font family */
#define FIXEDFONT	(1<<12)	/* font family */

#define ROMAN		(1<<13)	/* font style */
#define BOLD		(1<<14)	/* font style */
#define ITALIC		(1<<15)	/* font style */
#define BOLDITALIC	(1<<16)	/* font style */
#define OBLIQUE		(1<<17)	/* font style */
#define BOLDOBLIQUE	(1<<18)	/* font style */

#define ALIGN_CENTER	0	/* text alignment */
#define ALIGN_LEFT	(1<<0)	/* text alignment */
#define ALIGN_RIGHT	(1<<1)	/* text alignment */
#define ALIGN_ABOVE	(1<<2)	/* text alignment */
#define ALIGN_BELOW	(1<<3)	/* text alignment */

#define DEFAULT_VIEW_LONGITUDE	45
#define DEFAULT_VIEW_COLATITUDE	45

/* note that 1<<1 is actually 2 */
#define THICKNESS1	(1<<1)	/* line thickness */
#define THICKNESS2	(1<<2)	/* line thickness */
#define THICKNESS3	(1<<3)	/* line thickness */
#define THICKNESS4	(1<<4)	/* line thickness */
#define THICKNESS5	(1<<5)	/* line thickness */
#define THICKNESS6	(1<<6)	/* line thickness */
#define THICKNESS7	(1<<7)	/* line thickness */
#define THICKNESS8	(1<<8)	/* line thickness */
#define THICKNESS9	(1<<9)	/* line thickness */
#define THICKNESS10	(1<<10)	/* line thickness */

/* note that 1<<1 is actually 2 */
#define POINTSIZE1	(1<<1)	/* point size */
#define POINTSIZE2	(1<<2)	/* point size */
#define POINTSIZE3	(1<<3)	/* point size */
#define POINTSIZE4	(1<<4)	/* point size */
#define POINTSIZE5	(1<<5)	/* point size */
#define POINTSIZE6	(1<<6)	/* point size */
#define POINTSIZE7	(1<<7)	/* point size */
#define POINTSIZE8	(1<<8)	/* point size */
#define POINTSIZE9	(1<<9)	/* point size */
#define POINTSIZE10	(1<<10)	/* point size */


/* Output file formats
   IMPORTANT: if you alter the content/order of entries in this structure,
   adjust the exts[] array in file save_graph.c accordingly.
*/
enum PLOTOUT_FORMAT {
	JPEG_FORMAT,		/* jpeg */
	TIFF_FORMAT,		/* tiff */
	PPM6_FORMAT,		/* binary ppm */
	PPM3_FORMAT,		/* ascii ppm */
	PS_FORMAT,		/* postscript */
	CPS_FORMAT,		/* color postscript */
	VPS_FORMAT,		/* vector postscript */
	PNG_FORMAT		/* png */
};

#define DEFAULT_FONTFAMILY	HELVETICA
#define DEFAULT_FONTSTYLE	ROMAN
#define DEFAULT_FONTSIZE	10

#define DEFAULT_AXESFONTFAMILY	HELVETICA
#define DEFAULT_AXESFONTSTYLE	ROMAN
#define DEFAULT_AXESFONTSIZE	10

#define DEFAULT_LABELFONTFAMILY	HELVETICA
#define DEFAULT_LABELFONTSTYLE	ROMAN
#define DEFAULT_LABELFONTSIZE	12

#define DEFAULT_TITLEFONTFAMILY	HELVETICA
#define DEFAULT_TITLEFONTSTYLE	BOLDOBLIQUE
#define DEFAULT_TITLEFONTSIZE	14

#define DEFAULT_THICKNESS	THICKNESS1
#define DEFAULT_POINTSIZE	POINTSIZE3

#define DEFAULT_SPACECURVE_POINTS 60
#define MIN_SPACECURVE_POINTS	3

#define DEFAULT_TUBE_POINTS	16
#define MIN_TUBE_POINTS		3
#define DEFAULT_TUBE_RADIUS	1.0

#define DEFAULT_GRIDSIZE	25
#define MIN_GRIDSIZE		3
#define DEFAULT_XTICKS		5
#define DEFAULT_YTICKS		5
#define DEFAULT_ZTICKS		5

#define DEFAULT_CONTOURS	10

#define LIGHT0_AMBIENT_DEFAULT	0.75
#define LIGHT0_AMBIENT_MIN	0.25
#define LIGHT0_AMBIENT_MAX	0.95

#define LIGHT0_DIFFUSE_DEFAULT	0.85
#define LIGHT0_DIFFUSE_MIN	0.6
#define LIGHT0_DIFFUSE_MAX	1.0

#define MINZOOM			0.25
#define MAXZOOM			3.00

#define PROMPT1			"> "
#define PROMPT2			"  "

#define PLOTOUT_FILE_BASENAME	"dgplot"

typedef struct Expr {
	char		*lval;		/* left value, if an assignment */
	char		*rval;		/* symbolic form of the expression */
	struct Expr 	*fp;		/* next saved symbol in symtab */
} Expr;

typedef struct Range {	/* holds a range spec of the form x=lo..hi */
	char *id;       /* the "x" part */
	Expr *lo;       /* the "lo" part */
	Expr *hi;       /* the "hi" part */
} Range;

typedef struct Expr3 {
	Expr *u;
	Expr *v;
	Expr *w;
} Expr3;

typedef struct Point {
        double x;
        double y;
        double z;
} Point;

typedef struct polygonvertex {
	Expr3 *v;			/* a polygon vertex */
	Point numeric;			/* numerical value of the above */
	struct polygonvertex *fp;	/* next vertex */
} PolygonVertex;

typedef struct Fontdesc {
	unsigned int fontcode;
	XFontStruct *xfontinfo;
	GLuint glbase;
	GLuint nchars;
	short ascent;
	short descent;
	struct Fontdesc *fp;
} Fontdesc;

typedef struct TextObj {
	char *text;
	Expr *xspec;
	Expr *yspec;
	Expr *zspec;
	double x, y, z;
	struct TextObj *fp;
} TextObj;

typedef struct Plot3Dattribs {
	char		*id;		/* name of this set of plots */
	Range		*xrange;	/* x=[lo..hi] */
	Range		*yrange;	/* y=[lo..hi] */
	Expr		*xgrid;		/* grid=[xgrid,ygrid] */
	Expr		*ygrid;		/* grid=[xgrid,ygrid] */
	int		n, m;		/* numeric values of xgrid, ygrid */
	Expr		*num_points;	/* no. of pts on a curve or surface */
	Expr		*num_contours;	/* no. of contour lines */
	int		contours;	/* numerical value of the above */
	Expr		*tube_points;	/* no. of pts around a tubeplot */
	Expr		*tube_radius;	/* tubeplot radius */
	double		tuberadius;	/* numeric value of the above */
	GLuint		idx_overall;	/* GL displaylist for the full scene */
	GLuint		idx_objects;	/* GL displaylist for shown objects */
	GLuint		idx_bbox;	/* GL displaylist for bounding box */
	GLuint		idx_title;	/* GL displaylist for title  */
	char		*color;		/* X server color name */
	float		rgbcolors[4];	/* values of the above: in 0.0 to 1.0 */
        char		*xlabel;	/* x axes label */
        char		*ylabel;	/* y axes label */
        char		*zlabel;	/* z axes label */
        char 		*title;		/* title */
	unsigned int	titlefont;	/* title font */
        int 		titlewidth;	/* title width in pixels */
	Expr		*view_longitude;/*  view=[long,co-lat] */
	Expr		*view_colatitude;/* view=[long,co-lat] */
	double		longitude;	/* numeric values of */
	double		colatitude;	/* the above */
	unsigned int 	axes;		/* BOXED, NORMAL, FRAME, NOAXES */
	unsigned int	gridstyle;	/* TRIANGULAR_GRIDLINES, XY_GRIDLINES, etc */
	unsigned int	scaling;	/* CONSTRAINED/UNCONSTRAINED */
	unsigned int	shading;	/* XYZ, XY, Z, ZGREYSCALE, ZHUE, NONE */
	unsigned int	style;		/* PATCH, HIDDEN, etc. */
	unsigned int	coords;		/* coordinates choice */
	unsigned int	thickness;	/* line thickness */
	unsigned int	pointsize;	/* point size */
	unsigned int	font;		/* font for text objects */
	unsigned int	axesfont;	/* font for tick marks labels */
	unsigned int	labelfont;	/* font for axis labels */
	int		xticks;		/* no of x axis tickmarks */
	int		yticks;		/* no of y axis tickmarks */
	int		zticks;		/* no of z axis tickmarks */
	Expr		*x_viewmin, *x_viewmax;	/* view = ... bounds */
	Expr		*y_viewmin, *y_viewmax;	/* view = ... bounds */
	Expr		*z_viewmin, *z_viewmax;	/* view = ... bounds */
	double		xviewmin, xviewmax;	/* numerical values */
	double		yviewmin, yviewmax;	/* of the */
	double		zviewmin, zviewmax;	/* above */
	double		xmin, xmax;	/* computed bounds in the x direction */
	double		ymin, ymax;	/* computed bounds in the y direction */
	double		zmin, zmax;	/* computed bounds in the z direction */
	double		cx, cy, cz;	/* the "centroid" */
	double		xscale;		/* scale factor for */
	double		yscale;		/* constrained and  */
	double		zscale;		/* unconstrained rendering */
	double		radius;		/* radius, measured from centroid */
	int		nplots;		/* number of graphs to plot */
} Plot3Dattribs;

typedef struct Plot3D {
	unsigned int	type;		/* function plot or parametric plot */
	TextObj		*textobj;	/* textplot arguments */
	unsigned int	textalign;	/* alignment of textplot text */
	Expr3		*parg;		/* expression(s) to plot (symbolic) */
	PolygonVertex*	polygon_vertices; /* linked list of polygon vertices */
	Point		polygon_normal;	/* unit normal to a polygon */
	Point		**mesh;		/* coords of mesh points (numeric) */
	Point		**normal;	/* coords of normal vectors (numeric) */
	Point		*curvepoints;	/* coords of spacecurve pts (numeric */
	GLuint		idx;		/* GL displaylist index */
	double		xmin, xmax;	/* bounds in the x direction */
	double		ymin, ymax;	/* bounds in the y direction */
	double		zmin, zmax;	/* bounds in the z direction */
	Plot3Dattribs	*attr;		/* plot attributes */
	struct Plot3D	*fp;		/* next object in the scene */
	struct Plot3D	*bp;		/* prev object in the scene */
	struct Plot3D	*next;		/* next saved plot */
} Plot3D;

/* structure to hold information for the "savegraph(fmt, id, file)" command */
typedef struct SavegraphData {
	enum PLOTOUT_FORMAT fmt;	/* format of the saved file */
	char *file;			/* name of the file to save in */
	Plot3D *plot3d;			/* structure to save */
} SavegraphData;

Expr		*getsym(char *s);
void		create_form(void);
void		show_form(void);
Fontdesc	*create_fontdesc(void);
Fontdesc	*generate_font(unsigned int fontcode);
Fontdesc	*getfont(unsigned int fontcode);
Plot3D		*copy_plotlist(Plot3D *p);
Plot3D		*create_plot3d_structure(void);
Plot3D		*getplot(char *id);
Plot3D		*make_data_plot(char *filename, Plot3Dattribs *a);
Plot3D		*make_merged_plot(char *dispargs, Plot3Dattribs *a);
Plot3Dattribs	*copy_plot3d_attribs_structure(Plot3Dattribs *a);
Plot3Dattribs	*create_plot3d_attribs_structure(void);
Point		**create_mesh(int m, int n);
Point		**maketube(Point *p, int n, int m, double r);
Point		*create_curvepoints(int m);
Range		*copy_range(Range *r);
TextObj		*create_textobject(void);
char		*copy_string(char *s);
char		*expr_eval(char *s);
int		antialias(void);
int		canvas_expose(FL_OBJECT *obj, Window win, int w, int h, XEvent *xev, void *ud);
int		canvas_resize(FL_OBJECT *obj, Window win, int w, int h, XEvent *xev, void *ud);
int		evalf(Expr *e, double *d);
int		evaluate_plots(Plot3D *p);
int		grabmove(FL_OBJECT *obj, Window win, int w, int h, XEvent *xev, void *ud);
int		hoc(FILE *fp, double *arr);
int		idle_callback(XEvent *ev, void *data);
int		yylex(void);
int		yyparse(void);
void		assign_sym(Expr *s);
void		autorotate(void);
void		call_exit(void);
void		compile_boundingbox(Plot3Dattribs *a);
void		compile_scene(Plot3D *p);
void		compile_title(Plot3Dattribs *a);
void		compute_bounds(Plot3D *p);
void		compute_centroid(Plot3Dattribs *a);
void		compute_curvebounds(Plot3D *p);
void		compute_normals(Plot3D *p);
void		construct_contours(Plot3D *p, GLfloat *color);
void		create_displaylists(Plot3D *p);
void		create_menus(void);
void		create_object(Plot3D *p);
void		delplot(char *id);
void		display(void);
void		dump_symtab(void);
void		get_PostscriptFont(unsigned int fontcode, char **fontname, int *fontsize);
void		hide_about_form(FL_OBJECT *obj, long arg);
void		free_expr(Expr *s);
void		free_mesh(Point **mesh);
void		free_plot3d_attribs_structure(Plot3Dattribs *a);
void		free_plotlist(Plot3D *p);
void		generate_default_fonts(void);
void		initialize_glCanvas(void);
void		install_plot(Plot3D *p);
void		print_string(const char *s, GLuint base);
void		process_command(void);
void		putplot(Plot3D *p);
void		re_start(void);
void		read_from_file(char *filename);
void		remove_canvas_handlers(void);
void		reset_glCanvas(Plot3D *p);
void		save_graph_to_file(SavegraphData *sgd);
void		save_image_via_netpbm(SavegraphData *sgd, int as_shown);
void		set_menu_options(void);
void		set_vertex_color(Plot3D *p, double x, double y, double z);
void		show_about_form(FL_OBJECT *obj, long arg);
void		show_help(char *s);
void		show_parse_error(void);
void		show_sym(char *id);
void		write_jpeg(char *file, unsigned char *image, unsigned int w, unsigned int h);
void		write_png(char *file, unsigned char *image, unsigned int w, unsigned int h);
void		write_ppm3(char *file, unsigned char *image, unsigned int w, unsigned int h);
void		write_ppm6(char *file, unsigned char *image, unsigned int w, unsigned int h);
void		write_tiff(char *file, unsigned char *image, unsigned int w, unsigned int h);
void		write_ps(char *file, unsigned char *image, unsigned int w, unsigned int h);
void		write_cps(char *file, unsigned char *image, unsigned int w, unsigned int h);
void		write_vps(char *file);


#endif /* DG_DYNAGRAPH_H */
