#ifndef gl2psH
#define gl2psH

#include <stdio.h>
#include <GL/gl.h>
#include "bsp.h"

#define GL2PS_SET_POINT_SIZE	123
#define GL2PS_SET_LINE_WIDTH	456

#define GL2PS_DEFAULT_POINTSIZE		3.0
#define GL2PS_DEFAULT_LINEWIDTH		1.0

typedef enum {
	gl2psNO_SORT,
	gl2psSIMPLE_SORT,
	gl2psBSP_SORT,
	gl2psSIMPLE_AND_BSP_SORT
} gl2psSortMode;

typedef enum {
	gl2psGRAY,
	gl2psCOLOR
} gl2psShadeMode;

typedef struct {
	GLfloat *buf;			/* the feedback buffer */
	GLint buflen;			/* buffer length actually written */
	bspNode *rootnode;		/* root of the BSP tree */
	unsigned count_before[4];	/* each arrays must be as long as */
	unsigned count_after[4];	/*   entries in enum bspObjType ! */
	const char *filename;		/* name of output file */
	FILE *stream;			/* stream associated with above */
	gl2psSortMode sortmode;		/* sort mode */
	gl2psShadeMode shademode;	/* shade mode (gray/color) */
	float smoothness;		/* called by postscript's shfill */
	short int verbose;
	GLfloat mindepth, maxdepth;
	GLfloat linewidth;
} gl2psContext;


struct gl2psArgs {
	const char *file;
	gl2psSortMode sortmode;
	gl2psShadeMode shademode;
	float accuracy;
	float smoothness;
	short int verbose;
};

void gl2psBegin(struct gl2psArgs args);
GLint gl2psEnd(void);
void *gl2psMalloc(size_t nbytes);
GLvoid gl2psPointSize(GLfloat size);
GLvoid gl2psLineWidth(GLfloat width);
GLvoid gl2psText(const char *str, const char *fontname, GLint fontsize);

#endif /* gl2psH */
