%{

/* uncomment for dubugging
#define YYDEBUG 1
int yydebug = 1;
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "dynagraph.h"
#include "parse.h"

/* local function declarations */
static int  yyerror(char *s);
static char *catargs(char *a, char *b);
static Expr *merge_expr0(char *s);
static Expr *merge_expr1(Expr *e, short int type);
static Expr *merge_expr2(Expr *a, Expr *b, char op);
static Expr *merge_expr3(Expr *e, char *id);
static Expr *merge_expr4(Expr *e1, Expr *e2, char *id);
static PolygonVertex *dg_vertex_tail;

extern char *prognam;

/* Global variables, containing the results of sympolic parsing of the input */
extern Plot3D		*dg_plots;
extern Plot3Dattribs	*dg_attribs;
extern char		*dg_dispargs;
extern Expr3		*dg_vertex;	/* linked list of polygon vertices */

%}

%union {
	char			*str;
	Range			*rspec;
	unsigned int		fspec;
	Expr			*e;
	Expr3			*e3;
	enum PLOTOUT_FORMAT	fmt;
	SavegraphData		*sgd;
	PolygonVertex		*pvp;
}

%token		_DOTS _READ _DUMP _RESTART
%token		_SAVEGRAPH _JPEG _TIFF _PPM3 _PPM6 _POSTSCRIPT _COLORPOSTSCRIPT _VECTORPOSTSCRIPT _PNG
%token		_ALIGN _LEFT _RIGHT _ABOVE _BELOW
%token		_PLOT _DISPLAY _SPACECURVE _TUBEPLOT _POLYGONPLOT _DATAPLOT _TEXTPLOT
%token		_GRID _TITLE _LABELS _THICKNESS _POINTSIZE _VIEW _COLOR
%token		_CONTOURS _NUMPOINTS _TUBEPOINTS _TUBERADIUS _ORIENTATION _TICKMARKS
%token		_COORDS _CARTESIAN _CYLINDRICAL _Z_CYLINDRICAL _SPHERICAL
%token		_GRIDSTYLE _TRIANGULAR_GRIDLINES _XY_GRIDLINES _X_GRIDLINES _Y_GRIDLINES
%token		_STYLE _POINT _HIDDEN _PATCH _WIREFRAME
%token		_CONTOUR _PATCHNOGRID _PATCHCONTOUR
%token		_SCALING _CONSTRAINED _UNCONSTRAINED
%token		_AXES _BOXED _NORMAL _FRAME _NONE
%token		_SHADING _XYZ _XY _Z _ZGRAYSCALE _ZHUE
%token		_AXESFONT _FONT _LABELFONT _TITLEFONT
%token		_TIMES _COURIER _HELVETICA _SYMBOL 
%token		_ROMAN _BOLD _ITALIC _BOLDITALIC _OBLIQUE _BOLDOBLIQUE
%token	<str>	_NUMBER _IDENT _STRING
%type	<str>	readcmd string_or_ident
%type	<rspec>	range
%type	<e>	expr expr_assign
%type	<e3>	expr3 plot_desc plot_obj curve_obj
%type	<fspec>	fontspec times_fontstyle courier_fontstyle helvetica_fontstyle 
%type	<fmt>	savegraphformat;
%type	<sgd>	savegraphcmd;
%type	<pvp>	vertex vertex_chain;

%left ','		/* concatanes options */
%left '+' '-'		/* left associative, same precedence */
%left '*' '/'		/* left associative, higher precedence */
%right  '^'
%left UMINUS UPLUS

%%	/* begin grammar */

stmt	: /* empty */		     {	PRINTF(("empty statment 1\n")); }
	| stmt			term {	PRINTF(("empty statment 2\n")); }
	| stmt readcmd		term {	read_from_file($2); return PP_READ; }
	| stmt savegraphcmd	term {	save_graph_to_file($2); return PP_SAVEGRAPH; }
	| stmt _IDENT		term {	show_sym($2); return PP_SHOW; }
	| stmt _DUMP		term {	dump_symtab(); return PP_DUMP; }
	| stmt _RESTART		term {	re_start(); return PP_RESTART; }
	| stmt expr_assign	term {	assign_sym($2); return PP_ASGN; }
	| stmt plot_command	term {	return PP_PLOT; }
	| stmt disp_command	term {	return PP_DISPLAY; }
	| stmt curveplot_cmd	term {	return PP_SPACECURVE; }
	| stmt tubeplot_cmd	term {	return PP_TUBEPLOT; }
	| stmt polygonplot_cmd	term {	return PP_POLYGONPLOT; }
	| stmt textplot_cmd	term {	return PP_TEXTPLOT; }
	| stmt dataplot_cmd	term {	return PP_DATAPLOT; }
	| stmt plot_assign	';' {	return PP_PLOT_SAVE_SHOW; }
	| stmt plot_assign	':' {	return PP_PLOT_SAVE; }
	| stmt disp_assign	';' {	return PP_DISP_SAVE_SHOW; }
	| stmt disp_assign	':' {	return PP_DISP_SAVE; }
	| stmt dataplot_assign	';' {	return PP_DATAPLOT_SAVE_SHOW; }
	| stmt dataplot_assign	':' {	return PP_DATAPLOT_SAVE; }
	| stmt error		term {	yyerrok; return PP_ERROR; }
	;

term		: ';'
		| ':'
		;

readcmd		: _READ string_or_ident		{ $$=$2; }
		| _READ '(' string_or_ident ')'	{ $$=$3; }
		;


savegraphcmd	: _SAVEGRAPH '(' savegraphformat ')' {
				$$ = (SavegraphData *)malloc(sizeof(SavegraphData));
				$$->fmt = $3;
				$$->file = NULL; }
		| _SAVEGRAPH '(' savegraphformat ',' string_or_ident ')' {
				$$ = (SavegraphData *)malloc(sizeof(SavegraphData));
				$$->fmt = $3;
				$$->file = $5; }
		;

savegraphformat	: _JPEG			{ $$ = JPEG_FORMAT; }
		| _TIFF			{ $$ = TIFF_FORMAT; }
		| _PNG			{ $$ = PNG_FORMAT; }
		| _PPM3			{ $$ = PPM3_FORMAT; }
		| _PPM6			{ $$ = PPM6_FORMAT; }
		| _POSTSCRIPT		{ $$ = PS_FORMAT; }
		| _COLORPOSTSCRIPT	{ $$ = CPS_FORMAT; }
		| _VECTORPOSTSCRIPT	{ $$ = VPS_FORMAT; }
		;

expr_assign	: _IDENT ':' '=' expr {
			$4->lval = $1;
			$4->fp = NULL;
			$$ = $4; }
		;

range		: _IDENT '=' expr _DOTS expr {
			$$ = (Range *)malloc(sizeof(Range));
			$$->id=$1;
			$$->lo=$3;
			$$->hi=$5; }
		;

optlist		: option
		| optlist ',' option
		;

option		: grid
		| orientation
		| tickmarks
		| title
		| labels
		| thickness
		| pointsize
		| coords
		| shading
		| style
		| color
		| gridstyle
		| axes
		| scaling
		| axesfont
		| font
		| labelfont
		| titlefont
		| numpoints
		| contours
		| tubepoints
		| tuberadius
		| view
		| align
		;

grid		: _GRID '=' '[' expr ',' expr ']' {
			dg_attribs->xgrid=$4;
			dg_attribs->ygrid=$6; }
		;

orientation	: _ORIENTATION '=' '[' expr ',' expr ']' {
			dg_attribs->view_longitude=$4;
			dg_attribs->view_colatitude=$6; }
		;

tickmarks	: _TICKMARKS '=' '[' _NUMBER ',' _NUMBER ',' _NUMBER ']' {
			dg_attribs->xticks = atoi($4);
			dg_attribs->yticks = atoi($6);
			dg_attribs->zticks = atoi($8); }
		;


title		: _TITLE '=' string_or_ident	{ dg_attribs->title=$3; }
		;

labels		: _LABELS '=' '['
			string_or_ident { dg_attribs->xlabel=$4; } ','
			string_or_ident { dg_attribs->ylabel=$7; } ','
			string_or_ident { dg_attribs->zlabel=$10; } ']'
		;

thickness	: _THICKNESS '=' _NUMBER	{
			int thickness = atoi($3);
			/* thickness should be betweem 0 and 10 */
			thickness = MAX(0, thickness);
			thickness = MIN(10, thickness);
			/* convert to dynagraph's internal representation */
			if (thickness==0)
				dg_attribs->thickness = DEFAULT_THICKNESS;
			else
				dg_attribs->thickness = (1<<thickness);
			dg_attribs->thickness |= locked; }
		;

pointsize	: _POINTSIZE '=' _NUMBER	{
			int pointsize = atoi($3);
			/* pointsize should be betweem 0 and 10 */
			pointsize = MAX(0, pointsize);
			pointsize = MIN(10, pointsize);
			/* convert to dynagraph's internal representation */
			if (pointsize==0)
				dg_attribs->pointsize = DEFAULT_POINTSIZE;
			else
				dg_attribs->pointsize = (1<<pointsize);
			dg_attribs->pointsize |= locked; }
		;

coords		: _COORDS '=' coordsopt
		;

coordsopt	: _CARTESIAN	{ dg_attribs->coords=CARTESIAN; }
		| _CYLINDRICAL	{ dg_attribs->coords=CYLINDRICAL; }
		| _Z_CYLINDRICAL{ dg_attribs->coords=Z_CYLINDRICAL; }
		| _SPHERICAL	{ dg_attribs->coords=SPHERICAL; }
		;

shading		: _SHADING '=' shadingopt
		;

shadingopt
		: _XYZ		{ dg_attribs->shading=XYZSHADING|locked; }
		| _XY		{ dg_attribs->shading=XYSHADING|locked; }
		| _Z		{ dg_attribs->shading=ZSHADING|locked; }
		| _ZGRAYSCALE	{ dg_attribs->shading=ZGRAYSCALE|locked; }
		| _ZHUE		{ dg_attribs->shading=ZHUE|locked; }
		| _NONE		{ dg_attribs->shading=NOSHADING|locked; }
		;

style		: _STYLE '=' styleopt
		;

styleopt	: _POINT	{ dg_attribs->style=POINT|locked; }
		| _HIDDEN	{ dg_attribs->style=HIDDEN|locked; }
		| _PATCH	{ dg_attribs->style=PATCH|locked; }
		| _WIREFRAME	{ dg_attribs->style=WIREFRAME|locked; }
		| _CONTOUR	{ dg_attribs->style=CONTOUR|locked; }
		| _PATCHNOGRID	{ dg_attribs->style=PATCHNOGRID|locked; }
		| _PATCHCONTOUR	{ dg_attribs->style=PATCHCONTOUR|locked; }
		;

color		: _COLOR '=' string_or_ident	{ dg_attribs->color=$3; }
		;

gridstyle	: _GRIDSTYLE '=' gridstyleopt
		;

gridstyleopt	: _XY_GRIDLINES	{ dg_attribs->gridstyle=XY_GRIDLINES|locked; }
		| _X_GRIDLINES	{ dg_attribs->gridstyle=X_GRIDLINES|locked; }
		| _Y_GRIDLINES	{ dg_attribs->gridstyle=Y_GRIDLINES|locked; }
		| _TRIANGULAR_GRIDLINES	{ dg_attribs->gridstyle=TRIANGULAR_GRIDLINES|locked; }
		;

axes		: _AXES '=' axesopt
		;

axesopt		: _BOXED	{ dg_attribs->axes=BOXED|locked; }
		| _NORMAL	{ dg_attribs->axes=NORMAL|locked; }
		| _FRAME	{ dg_attribs->axes=FRAME|locked; }
		| _NONE		{ dg_attribs->axes=NOAXES|locked; }
		;

align		: _ALIGN '=' alignopt
		;

alignopt	: _LEFT		{ dg_plots->textalign |= ALIGN_LEFT; }
		| _RIGHT	{ dg_plots->textalign |= ALIGN_RIGHT; }
		| _ABOVE	{ dg_plots->textalign |= ALIGN_ABOVE; }
		| _BELOW	{ dg_plots->textalign |= ALIGN_BELOW; }
		;

scaling		: _SCALING '=' scalingopt
		;

scalingopt	: _CONSTRAINED
			{ dg_attribs->scaling=CONSTRAINED|locked; }
		| _UNCONSTRAINED
			{ dg_attribs->scaling=UNCONSTRAINED|locked; }
		;

view		: _VIEW '=' expr _DOTS expr {
			dg_attribs->z_viewmin = $3;
			dg_attribs->z_viewmax = $5; }
		| _VIEW '=' '[' expr _DOTS  expr ',' expr _DOTS expr ',' expr _DOTS expr ']' {
			dg_attribs->x_viewmin = $4;
			dg_attribs->x_viewmax = $6;
			dg_attribs->y_viewmin = $8;
			dg_attribs->y_viewmax = $10;
			dg_attribs->z_viewmin = $12;
			dg_attribs->z_viewmax = $14; }
		;

numpoints	:  _NUMPOINTS '=' expr		{ dg_attribs->num_points=$3; }
		;

contours	:  _CONTOURS '=' expr		{ dg_attribs->num_contours=$3; }
		;

tubepoints	:  _TUBEPOINTS '=' expr		{ dg_attribs->tube_points=$3; }
		;

tuberadius	:  _TUBERADIUS '=' expr		{ dg_attribs->tube_radius=$3; }
		;

axesfont	: _AXESFONT '=' fontspec
			{ dg_attribs->axesfont = $3|locked; } 
		;

font		: _FONT '=' fontspec
			{ dg_attribs->font = $3|locked; }
		;

labelfont	: _LABELFONT '=' fontspec
			{ dg_attribs->labelfont = $3|locked; }
		;

titlefont	: _TITLEFONT '=' fontspec
			{ dg_attribs->titlefont = $3|locked; }
		;

fontspec	: '[' _TIMES ',' times_fontstyle ',' _NUMBER ']' {
			$$ = TIMES + $4 + MIN(MAX(1,atoi($6)), 0177); }
		| '[' _COURIER ',' courier_fontstyle ',' _NUMBER ']' {
			$$ = COURIER + $4 + MIN(MAX(1,atoi($6)), 0177); }
		| '[' _COURIER ',' _NUMBER ']' {
			$$ = COURIER + ROMAN + MIN(MAX(1,atoi($4)), 0177); }
		| '[' _HELVETICA ',' helvetica_fontstyle ',' _NUMBER ']' {
			$$ = HELVETICA + $4 + MIN(MAX(1,atoi($6)), 0177); }
		| '[' _HELVETICA ',' _NUMBER ']' {
			$$ = HELVETICA + ROMAN + MIN(MAX(1,atoi($4)), 0177); }
		| '[' _SYMBOL ',' _NUMBER ']' {
			$$ = SYMBOL + ROMAN + MIN(MAX(1,atoi($4)), 0177); }
		;

times_fontstyle	: _ROMAN	{ $$ = ROMAN; }
		| _BOLD		{ $$ = BOLD; }
		| _ITALIC	{ $$ = ITALIC; }
		| _BOLDITALIC	{ $$ = BOLDITALIC; }
		;

courier_fontstyle: _BOLD	{ $$ = BOLD; }
		| _OBLIQUE	{ $$ = OBLIQUE; }
		| _BOLDOBLIQUE	{ $$ = BOLDOBLIQUE; }
		;

helvetica_fontstyle: _BOLD	{ $$ = BOLD; }
		| _OBLIQUE	{ $$ = OBLIQUE; }
		| _BOLDOBLIQUE	{ $$ = BOLDOBLIQUE; }
		;

plot_assign	: _IDENT ':' '=' plot_command	{ dg_attribs->id = $1; }
		| _IDENT ':' '=' curveplot_cmd	{ dg_attribs->id = $1; }
		| _IDENT ':' '=' tubeplot_cmd	{ dg_attribs->id = $1; }
		| _IDENT ':' '=' polygonplot_cmd{ dg_attribs->id = $1; }
		| _IDENT ':' '=' textplot_cmd	{ dg_attribs->id = $1; }
		;

plot_command	: _PLOT '(' plot_fullarg ')'
		;

plot_fullarg	: plot_arg ',' range ',' range {
			dg_attribs->xrange=$3;
			dg_attribs->yrange=$5; }
		| plot_fullarg ',' optlist
		;

plot_arg	: plot_obj		{ /* empty */ }
		| plot_objset		{ /* empty */ }
		;

plot_objset	: '{' plot_objseq '}'
		;

plot_objseq	: plot_obj		{ /* empty */ }
		| plot_objseq ',' plot_obj {
			Plot3D *p, *q=NULL;
			for (p=dg_plots; p!=NULL; p=p->fp)
				q=p;
			p = create_plot3d_structure();
			p->attr = dg_attribs;
			p->type = $3->u==NULL ?  FUNCTION_PLOT : PARAM_PLOT;
			p->parg=$3;
			q->fp = p;
			p->bp = q; }
		;

plot_obj	: plot_desc {
			if (dg_plots==NULL) {
				dg_plots = create_plot3d_structure();
				dg_attribs = create_plot3d_attribs_structure();
				dg_plots->attr = dg_attribs;
				dg_plots->type = $1->u==NULL ?
					FUNCTION_PLOT : PARAM_PLOT;
				dg_plots->parg=$1;
			}
			$$=$1; }
		;

plot_desc	: expr	{
			$$=(Expr3 *)malloc(sizeof(Expr3));
			$$->u=NULL;
			$$->v=NULL;
			$$->w=$1; }
		| '[' expr3 ']'
			{ $$=$2; }
		;


polygonplot_cmd	: _POLYGONPLOT '(' polygonplot_fullarg ')'
		;

polygonplot_fullarg
		: vertex_list
		| vertex_list ',' optlist
		;

vertex_list	: '[' vertex_chain ']' {
				dg_plots = create_plot3d_structure();
				dg_attribs = create_plot3d_attribs_structure();
				dg_plots->attr = dg_attribs;
				dg_plots->type = POLYGON_PLOT;
				dg_plots->polygon_vertices = $2; }
		;

vertex_chain	: vertex	{ $$ = dg_vertex_tail =  $1; }
		| vertex_chain ',' vertex {
				dg_vertex_tail->fp = $3;
				dg_vertex_tail = $3;
				$$ = $1; }
		;

vertex		: '[' expr3 ']' {
				$$ = (PolygonVertex*)malloc(sizeof(PolygonVertex));
				$$->v = $2;
				$$->fp = NULL; }
		;

tubeplot_cmd	: _TUBEPLOT '(' curveplot_fullarg ')' {
				Plot3D *p;
				for (p=dg_plots; p!=NULL; p=p->fp)
					p->type = TUBE_PLOT; }
		;

curveplot_cmd	: _SPACECURVE '(' curveplot_fullarg ')'
		;

curveplot_fullarg: curveplot_arg ',' range	{ dg_attribs->xrange=$3; }
		| curveplot_fullarg ',' optlist
		;

curveplot_arg	: curve_obj		{ /* empty */ }
		| curve_objset		{ /* empty */ }
		;

curve_objset	: '{' curve_objseq '}'
		;
	
curve_objseq	: curve_obj		{ /* empty */ }
		| curve_objseq ',' curve_obj {
			Plot3D *p, *q=NULL;
			for (p=dg_plots; p!=NULL; p=p->fp)
				q=p;
			p = create_plot3d_structure();
			p->attr = dg_attribs;
			p->type = CURVE_PLOT;
			p->parg=$3;
			q->fp = p;
			p->bp = q; }
		;

curve_obj	: '[' expr3 ']'	{
			if (dg_plots==NULL) {
				dg_plots = create_plot3d_structure();
				dg_attribs = create_plot3d_attribs_structure();
				dg_plots->attr = dg_attribs;
				dg_plots->type = CURVE_PLOT;
				dg_plots->parg=$2;
			}
			$$=$2; }
		;

disp_assign	: _IDENT ':' '=' disp_command { dg_attribs->id = $1; }
		;

disp_command	: _DISPLAY '(' disp_fullarg ')'
		;

disp_fullarg	: disp_arg
		| disp_fullarg ',' optlist
		;

disp_arg	: disp_obj
		| disp_objset
		;

disp_objset	: '{' disp_objseq '}'
		| '[' disp_objseq ']'
		;

disp_objseq	: disp_obj
		| disp_objseq ',' disp_obj
		;

disp_obj	: _IDENT {
			if (dg_attribs==NULL) {
				dg_attribs = create_plot3d_attribs_structure();
				dg_dispargs=$1;
			} else
				dg_dispargs = catargs(dg_dispargs, $1); }
		;

dataplot_assign	: _IDENT ':' '=' dataplot_cmd { dg_attribs->id = $1; }
		;


dataplot_cmd	: _DATAPLOT '(' dataplot_fullarg ')'
		;

dataplot_fullarg: dataplot_arg
		| dataplot_fullarg ',' optlist
		;

dataplot_arg	: string_or_ident {
			dg_attribs = create_plot3d_attribs_structure();
			dg_dispargs=$1; }
		;

textplot_cmd	: _TEXTPLOT '(' textplot_fullarg ')'
		;

textplot_fullarg: textplot_arg
		| textplot_fullarg ',' optlist
		;

textplot_arg	: textplot_obj
		| textplot_objset
		;

textplot_objset	: '{' textplot_objseq '}'
		| '[' textplot_objseq ']'
		;

textplot_objseq	: textplot_obj
		| textplot_objseq ',' textplot_obj
		;

textplot_obj	: '[' expr ',' expr ',' expr ',' string_or_ident ']' {
			if (dg_plots==NULL) {
				dg_plots = create_plot3d_structure();
				dg_attribs = create_plot3d_attribs_structure();
				dg_plots->attr = dg_attribs;
				dg_plots->type = TEXT_PLOT;
				dg_plots->textobj = create_textobject();
			} else {
				TextObj *p = create_textobject();
				p->fp = dg_plots->textobj;
				dg_plots->textobj = p;
			}
			dg_plots->textobj->xspec = $2;
			dg_plots->textobj->yspec = $4;
			dg_plots->textobj->zspec = $6;
			dg_plots->textobj->text = $8; }
		;

expr3		: expr ',' expr ',' expr {
			$$=(Expr3 *)malloc(sizeof(Expr3));
			$$->u=$1;
			$$->v=$3;
			$$->w=$5; }
		;


string_or_ident	: _STRING		{ $$=$1; }
		| _IDENT		{ $$=$1; }
		;

 
expr		: _NUMBER		{ $$=merge_expr0($1); }
		| _IDENT		{ $$=merge_expr0($1); }
		| '-' expr %prec UMINUS	{ $$=merge_expr1($2, 0); }
		| '+' expr %prec UPLUS	{ $$=merge_expr1($2, 1); }
		| expr '+' expr		{ $$=merge_expr2($1, $3, '+'); }
		| expr '-' expr		{ $$=merge_expr2($1, $3, '-'); }
		| expr '*' expr		{ $$=merge_expr2($1, $3, '*'); }
		| expr '/' expr		{ $$=merge_expr2($1, $3, '/'); }
		| expr '^' expr		{ $$=merge_expr2($1, $3, '^'); }
		| '(' expr ')'		{ $$=merge_expr1($2, 2); }
		| _IDENT '(' expr ')'	{ $$=merge_expr3($3, $1); }
		| _IDENT '(' expr ',' expr ')'	{ $$=merge_expr4($3, $5, $1); }
		;

%%	/* end of grammar */


static Expr *merge_expr0(char *s)
{
	Expr *e;
	e = malloc(sizeof(Expr));
	e->lval = NULL;
	e->rval = s;
	e->fp = NULL;
	return e;
}

static Expr *merge_expr1(Expr *e, short int type)
{
	char *s;

	if (type==0) {				/* expr = - expr */
		s = malloc(strlen(e->rval)+2);
		sprintf(s, "%c%s", '-', e->rval);
	} else if (type==1) {			/* expr = + expr */
		s = malloc(strlen(e->rval)+2);
		sprintf(s, "%c%s", '+', e->rval);
	} else /* type==2 */ {			/* expr = ( expr ) */
		s = malloc(strlen(e->rval)+3);
		sprintf(s, "(%s)", e->rval);
	}
	free(e->rval);
	e->rval = s;
	return e;
}

static Expr *merge_expr2(Expr *a, Expr *b, char op)
{
	Expr *e;

	e = malloc(sizeof(Expr));
	e->lval = NULL;
	e->rval = malloc(strlen(a->rval)+strlen(b->rval)+2);
	sprintf(e->rval, "%s%c%s", a->rval, op, b->rval);
	e->fp = NULL;
	
	free_expr(a);
	free_expr(b);

	return e;
}

static Expr *merge_expr3(Expr *e, char *id)
{
	char *s;

	s = malloc(strlen(e->rval)+strlen(id)+3);
	sprintf(s, "%s(%s)", id, e->rval);
	free(e->rval);
	e->rval = s;
	free(id);
	return e;
}

static Expr *merge_expr4(Expr *e1, Expr *e2, char *id)
{
	char *s;

	s = malloc(strlen(e1->rval)+strlen(e2->rval)+strlen(id)+4);
	sprintf(s, "%s(%s,%s)", id, e1->rval,e2->rval);
	free(e1->rval);
	e1->rval = s;
	free_expr(e2);
	free(id);
	return e1;
}


/* crates a new comma-separated string from strings a and b */
static char *catargs(char *a, char *b)
{
	char *s;

	s = malloc(strlen(a)+strlen(b)+2);
	sprintf(s, "%s,%s", a, b);
	free(a);
	free(b);
	return s;
}

static int yyerror(char *s)
{
	show_parse_error(); 
	return -2;	/* not used */
}
