#include "hoc.h"
#include "hoc-parse.h"	/* automatically generated by yacc -d */
#include <math.h>

static struct {		/* Keywords */
	char	*name;
	int	kval;
} keywords[] = {
	{ "proc",	PROC	},
	{ "func",	FUNC	},
	{ "return",	RETURN	},
	{ "if",		IF	},
	{ "else",	ELSE	},
	{ "while",	WHILE	},
	{ "for",	FOR	},
	{ "print",	PRINT	},
	{ 0,		0	}
};

static struct {		/* Constants */
	char *name;
	double cval;
} consts[] = {
	{ "PI",	 	 3.14159265358979323846	},
	{ "Pi",	 	 3.14159265358979323846	},
	{ "E",	 	 2.71828182845904523536	},
	{ "GAMMA", 	 0.57721566490153286060	},	/* Euler */
	{ "DEG",	57.29577951308232087680	},	/* deg/radian */
	{ "PHI", 	 1.61803398874989484820	},	/* golden ratio */
	{ "PREC",	15,			},	/* output precision */
	{ 0,		 0			}
};

static double Heaviside(double x)
{
	return x<0.0 ? 0.0 : 1.0;
}

static double integer(double x)
{
	return (double)(long)x;
}

static struct {		/* Built-ins */
	char *name;
	double	(*func)(double);
} builtins[] = {
	{ "sin",	sin	},
	{ "cos",	cos	},
	{ "tan",	tan	},
	{ "atan",	atan	},
	{ "arctan",	atan	},
	{ "asin",	asin	},
	{ "arcsin",	asin	},
	{ "acos", 	acos	},
	{ "arccos", 	acos	},
	{ "sinh",	sinh	},
	{ "cosh",	cosh	},
	{ "tanh",	tanh	},
	{ "asinh",	asinh	},
	{ "arcsinh",	asinh	},
	{ "acosh",	acosh	},
	{ "arccosh",	acosh	},
	{ "atanh",	atanh	},
	{ "arctanh",	atanh	},
	{ "log",	log	},
	{ "ln",		log	},	/* "ln" and "log" are synonyms */
	{ "log10", 	log10	},
	{ "exp",	exp	},
	{ "sqrt",	sqrt	},
	{ "int",	integer	},
	{ "abs",	fabs	},
	{ "erf",	erf	},
	{ "erfc",	erfc	},
	{ "Heaviside",	Heaviside },
	{ 0,		0	}
};

static double besselj(double v, double x)
{
	return jn((int)v,x);
}

static double bessely(double v, double x)
{
	return yn((int)v,x);
}

static struct {		/* Built-ins of two variables, added by RR */
	char *name;
	double	(*func)(double,double);
} builtins2[] = {
	{ "atan2",	atan2	},
	{ "arctan2",	atan2	},
	{ "BesselJ",	besselj	},
	{ "BesselY",	bessely	},
	{ 0,		0	}
};

void init(void)	/* install constants and built-ins in table */
{
	int i;
	Symbol *s;
	for (i = 0; keywords[i].name; i++)
		install(keywords[i].name, keywords[i].kval, 0.0);
	for (i = 0; consts[i].name; i++)
		install(consts[i].name, VAR, consts[i].cval);
	for (i = 0; builtins[i].name; i++) {
		s = install(builtins[i].name, BLTIN, 0.0);
		s->u.ptr = builtins[i].func;
	}
	for (i = 0; builtins2[i].name; i++) {
		s = install(builtins2[i].name, BLTIN2, 0.0);
		s->u.ptr2 = builtins2[i].func;
	}
}
