#include <stdio.h>
#include <stdlib.h>	/* for strtod() */
#include <string.h>	/* for strchr() */
#include <ctype.h>
#include "hoc-parse-hack.h"
#include "hoc.h"
#include "hoc-parse.h"	/* automatically generated by yacc -d */

#include <signal.h>
#include <setjmp.h>
#include <errno.h>

extern	int	yyparse(void);
extern	int	errno;
extern	int	indef;
static	int	lineno = 1;
double	*retp;	/* pointer to the array of hoc's return values */
jmp_buf	begin;
char	*progname;
static	int c;
static	FILE *fin;

static int backslash(int c)	/* get next char with \'s interpreted */
{
	static char transtab[] = "b\bf\fn\nr\rt\t";
	if (c != '\\')
		return c;
	c = getc(fin);
	if (islower(c) && strchr(transtab, c))
		return strchr(transtab, c)[1];
	return c;
}

static int follow(int expect, int ifyes, int ifno) /* look ahead for >=, etc. */
{
	int c = getc(fin);

	if (c == expect)
		return ifyes;
	ungetc(c, fin);
	return ifno;
}

int yylex(void)
{
	while ((c=getc(fin)) == ' ' || c == '\t')
		;
	if (c == EOF)
		return 0;
	if (c == '\\') {
		c = getc(fin);
		if (c == '\n') {
			lineno++;
			return yylex();
		}
	}
	if (c == '#') {		/* comment */
		while ((c=getc(fin)) != '\n' && c != EOF)
			;
		if (c == '\n')
			lineno++;
		return c;
	}
	if (c == '.' || isdigit(c)) {	/* number */
		double d;
		ungetc(c, fin);
		fscanf(fin, "%lf", &d);
		yylval.sym = install("", NUMBER, d);
		return NUMBER;
	}
	if (isalpha(c) || c == '_') {
		Symbol *s;
		char sbuf[100], *p = sbuf;
		do {
			if (p >= sbuf + sizeof(sbuf) - 1) {
				*p = '\0';
				execerror("name too long", sbuf);
			}
			*p++ = c;
		} while ((c=getc(fin)) != EOF && (isalnum(c) || c == '_'));
		ungetc(c, fin);
		*p = '\0';
		if ((s=lookup(sbuf)) == 0)
			s = install(sbuf, UNDEF, 0.0);
		yylval.sym = s;
		return s->type == UNDEF ? VAR : s->type;
	}
	if (c == '$') {	/* argument? */
		int n = 0;
		while (isdigit(c=getc(fin)))
			n = 10 * n + c - '0';
		ungetc(c, fin);
		if (n == 0)
			execerror("strange $...", (char *)0);
		yylval.narg = n;
		return ARG;
	}
	if (c == '"') {	/* quoted string */
		char sbuf[100], *p;
		for (p = sbuf; (c=getc(fin)) != '"'; p++) {
			if (c == '\n' || c == EOF)
				execerror("missing quote", "");
			if (p >= sbuf + sizeof(sbuf) - 1) {
				*p = '\0';
				execerror("string too long", sbuf);
			}
			*p = backslash(c);
		}
		*p = 0;
		yylval.sym = (Symbol *)emalloc(strlen(sbuf)+1);
		strcpy((char*)yylval.sym, sbuf);
		return STRING;
	}
	switch (c) {
	case '+':	return follow('+', INC, follow('=', ADDEQ, '+'));
	case '-':	return follow('-', DEC, follow('=', SUBEQ, '-'));
	case '*':	return follow('=', MULEQ, '*');
	case '/':	return follow('=', DIVEQ, '/');
	case '%':	return follow('=', MODEQ, '%');
	case '>':	return follow('=', GE, GT);
	case '<':	return follow('=', LE, LT);
	case '=':	return follow('=', EQ, '=');
	case '!':	return follow('=', NE, NOT);
	case '|':	return follow('|', OR, '|');
	case '&':	return follow('&', AND, '&');
	case '\n':	lineno++; return '\n';
	default:	return c;
	}
}

void warning(char *s, char *t)		/* print warning message */
{
	fprintf(stderr, "%s: %s", progname, s);
	if (t)
		fprintf(stderr, " %s", t);
	fprintf(stderr, " near line %d\n", lineno);
}

void yyerror(char* s)	/* report compile-time error */
{
	execerror(s, (char *)0);
}

void execerror(char* s, char* t)	/* recover from run-time error */
{
/* don't want to see hoc warning messages -- RR
	warning(s, t);
*/
	longjmp(begin, 0);
}

static void fpecatch(int sig)		/* catch floating point exceptions */
{
	execerror("floating point exception", (char *) 0);
}

void defnonly(char *s)	/* warn if illegal definition */
{
	if (!indef)
		execerror(s, "used outside definition");
}

/* The main interface routine to hoc.

   The file fp should contain a hoc script.  The calculations are
   performed normally, but the "print" statements, which under standard
   hoc prints results to the screen, has been modified to save results
   in the array ret. which is assumed to be large enough to hold the
   entire output.

   On success, returns 1, else 0.
 */
int hoc(FILE *fp, double *ret)
{
	progname = "hoc";

	/* make args global */
	fin = fp;
	retp = ret;

	clear_symlist();
	lineno = 1;
	init();

	if (setjmp(begin)!=0) {
		fprintf(stderr, "HOC ABORTING\n");
		return 0;
	}

	signal(SIGFPE, fpecatch);
	for (initcode(); yyparse(); initcode())
		execute(progbase);

	return 1;
}
