#include <stdio.h>
#include <float.h>
#include <forms.h>
#include <GL/gl.h>
#include "formdefs.h"
#include "dynagraph.h"

extern GLfloat	*bgColor;
extern FD_form *Form;

GLfloat light0_diffuse[] = {
	LIGHT0_DIFFUSE_DEFAULT,
	LIGHT0_DIFFUSE_DEFAULT,
	LIGHT0_DIFFUSE_DEFAULT,
	ALPHA
};

GLfloat light0_ambient[] = {
	LIGHT0_AMBIENT_DEFAULT,
	LIGHT0_AMBIENT_DEFAULT,
	LIGHT0_AMBIENT_DEFAULT,
	ALPHA
};

GLfloat light0_specular[] = { 1.0, 1.0, 1.0, ALPHA };
GLfloat light0_position[] = { 86, 7.5, 50, 0.0 };
GLfloat light_ambient[]	= { 0.0, 0.0, 0.0, ALPHA };


/* this routine sets those parts of the initializations
   which depend on the interaction between Xforms and OpenGL
*/
static void more_initializations(void)
{
	FL_OBJECT *glCanvas = Form->glCanvas;

	if (fl_get_button(Form->light_button))
		glEnable(GL_LIGHTING);
	else
		glDisable(GL_LIGHTING);

	fl_add_canvas_handler(glCanvas, Expose, canvas_expose, NULL);
	fl_add_canvas_handler(glCanvas, ConfigureNotify, canvas_resize, NULL);
	fl_add_canvas_handler(glCanvas, ButtonPress, grabmove, NULL);
	fl_add_canvas_handler(glCanvas, ButtonRelease, grabmove, NULL);
	fl_add_canvas_handler(glCanvas, MotionNotify, grabmove, NULL);

	/* ingnore mouse movement if button is not pressed
	   (from chapter on Canvas in the xforms manual) */
	fl_remove_selected_xevent(FL_ObjWin(glCanvas),
			PointerMotionMask|PointerMotionHintMask);
}

void initialize_glCanvas(void)
{
	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, light_ambient);

	glLightfv(GL_LIGHT0, GL_DIFFUSE, light0_diffuse);
	glLightfv(GL_LIGHT0, GL_AMBIENT, light0_ambient);
	glLightfv(GL_LIGHT0, GL_SPECULAR, light0_specular);
	glLightfv(GL_LIGHT0, GL_POSITION, light0_position);
	glEnable(GL_LIGHT0);

	glClearColor(bgColor[0], bgColor[1], bgColor[2], bgColor[3]);
	glEnable(GL_DEPTH_TEST);


	/* These, togeter with glEnable/glDisable of
	   GL_LINE_SMOOTH, GL_POINT_SMOOTH, and GL_BLEND properties
	   produce antialiased points and lines.
	   
	   Antialiasing is not always desirable.  For one thing, it is
	   a very CPU intensive and slows moving objects considerably.
	   Moreover, it looks terrible on a grid imposed on a solid
	   surface.  It also looks very odd on a bounding box.  On the
	   other hand, it looks very nice on spacecurces and points.
	   So we use the effect very sparingly.
         */
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glHint(GL_LINE_SMOOTH_HINT, GL_DONT_CARE);
	glHint(GL_POINT_SMOOTH_HINT, GL_DONT_CARE);
	
	if (Form!=NULL)
		more_initializations();
}

void remove_canvas_handlers(void)
{
	FL_OBJECT *glCanvas = Form->glCanvas;
	fl_remove_canvas_handler(glCanvas, Expose, canvas_expose);
	fl_remove_canvas_handler(glCanvas, ConfigureNotify, canvas_resize);
	fl_remove_canvas_handler(glCanvas, ButtonPress, grabmove);
	fl_remove_canvas_handler(glCanvas, ButtonRelease, grabmove);
	fl_remove_canvas_handler(glCanvas, MotionNotify, grabmove);
}
