%{

#include <stdio.h>
#include <string.h>
#include "dynagraph.h"
#include "parse.h"
#include "grammar.h"

extern int restart;
static int nchars;

%}

D	[0-9]
E	(e[+-]?{D}+)
NUM	(({D}+|{D}+"."{D}*|{D}*"."{D}+){E}?)
ID	([A-Za-z][A-Za-z0-9_]*)
STR	(`[^`\n\003]*`|'[^'\n\003]*'|\"[^\"\n\003]*\")

%start GETAXES GETSTYLE GETSCALING GETSHADING GETFONT
%start GETGRIDSTYLE GETCOORDS GETALIGN GETFMT

%option noyywrap

%%

	if (restart) { nchars=0; restart=0; BEGIN(INITIAL); }

\003					{ return 0;		/* end of input */ }

<<EOF>>					{ return 0;		/* end of input */ }

[ \t\n]+				{ nchars+=yyleng;	/* skip whitespace */ }

#[^\n]*					{ nchars+=yyleng; 	/* eat up comment */ }

\.\.\.*					{ nchars+=yyleng; return _DOTS;	/* two or more dots */ }

<INITIAL>(plot|plot3d)			{ nchars+=yyleng; return _PLOT; }
<INITIAL>spacecurve			{ nchars+=yyleng; return _SPACECURVE; }
<INITIAL>tubeplot			{ nchars+=yyleng; return _TUBEPLOT; }
<INITIAL>polygonplot			{ nchars+=yyleng; return _POLYGONPLOT; }
<INITIAL>display			{ nchars+=yyleng; return _DISPLAY; }
<INITIAL>(dataplot|plotdata)		{ nchars+=yyleng; return _DATAPLOT; }
<INITIAL>(textplot|textplot3d)		{ nchars+=yyleng; return _TEXTPLOT; }
<INITIAL>read				{ nchars+=yyleng; return _READ; }
<INITIAL>dump				{ nchars+=yyleng; return _DUMP; }
<INITIAL>restart			{ nchars+=yyleng; return _RESTART; }
<INITIAL>savegraph			{ nchars+=yyleng; BEGIN(GETFMT); return _SAVEGRAPH; }

<GETFMT>(jpeg|jpg)			{ nchars+=yyleng; BEGIN(INITIAL); return _JPEG; }
<GETFMT>(tiff)				{ nchars+=yyleng; BEGIN(INITIAL); return _TIFF; }
<GETFMT>(png)				{ nchars+=yyleng; BEGIN(INITIAL); return _PNG; }
<GETFMT>(ppm|ppm6|ppm-raw|ppm-binary)	{ nchars+=yyleng; BEGIN(INITIAL); return _PPM6; }
<GETFMT>(ppm3|ppm-ascii)		{ nchars+=yyleng; BEGIN(INITIAL); return _PPM3; }
<GETFMT>(ps|postscript|postscript-gray)	{ nchars+=yyleng; BEGIN(INITIAL); return _POSTSCRIPT; }
<GETFMT>(cps|postscript-colou?r)	{ nchars+=yyleng; BEGIN(INITIAL); return _COLORPOSTSCRIPT; }
<GETFMT>(vps|postscript-vector)		{ nchars+=yyleng; BEGIN(INITIAL); return _VECTORPOSTSCRIPT; }

<INITIAL>grid				{ nchars+=yyleng; return _GRID; }
<INITIAL>view				{ nchars+=yyleng; return _VIEW; }
<INITIAL>(thickness|linewidth)		{ nchars+=yyleng; return _THICKNESS; }
<INITIAL>pointsize			{ nchars+=yyleng; return _POINTSIZE; }
<INITIAL>tuberadius			{ nchars+=yyleng; return _TUBERADIUS; }
<INITIAL>tubepoints			{ nchars+=yyleng; return _TUBEPOINTS; }
<INITIAL>numpoints			{ nchars+=yyleng; return _NUMPOINTS; }
<INITIAL>contours			{ nchars+=yyleng; return _CONTOURS; }
<INITIAL>title				{ nchars+=yyleng; return _TITLE; }
<INITIAL>(colou?r)			{ nchars+=yyleng; return _COLOR; }
<INITIAL>labels				{ nchars+=yyleng; return _LABELS; }
<INITIAL>orientation			{ nchars+=yyleng; return _ORIENTATION; }
<INITIAL>tickmarks			{ nchars+=yyleng; return _TICKMARKS; }

<INITIAL>axes				{ nchars+=yyleng; BEGIN(GETAXES); return _AXES; }
<GETAXES>(boxed|BOXED)			{ nchars+=yyleng; BEGIN(INITIAL); return _BOXED; }
<GETAXES>(normal|NORMAL)		{ nchars+=yyleng; BEGIN(INITIAL); return _NORMAL; }
<GETAXES>(frame|FRAME)			{ nchars+=yyleng; BEGIN(INITIAL); return _FRAME; }
<GETAXES>(none|NONE)			{ nchars+=yyleng; BEGIN(INITIAL); return _NONE; }

<INITIAL>style				{ nchars+=yyleng; BEGIN(GETSTYLE); return _STYLE; }
<GETSTYLE>(point|POINT)			{ nchars+=yyleng; BEGIN(INITIAL); return _POINT; }
<GETSTYLE>(hidden|HIDDEN)		{ nchars+=yyleng; BEGIN(INITIAL); return _HIDDEN; }
<GETSTYLE>(patch|PATCH)			{ nchars+=yyleng; BEGIN(INITIAL); return _PATCH; }
<GETSTYLE>(wireframe|line)		{ nchars+=yyleng; BEGIN(INITIAL); return _WIREFRAME; }
<GETSTYLE>(WIREFRAME|LINE)		{ nchars+=yyleng; BEGIN(INITIAL); return _WIREFRAME; }
<GETSTYLE>(contour|CONTOUR)		{ nchars+=yyleng; BEGIN(INITIAL); return _CONTOUR; }
<GETSTYLE>(patchnogrid|PATCHNOGRID)	{ nchars+=yyleng; BEGIN(INITIAL); return _PATCHNOGRID; }
<GETSTYLE>(patchcontour|PATCHCONTOUR)	{ nchars+=yyleng; BEGIN(INITIAL); return _PATCHCONTOUR; }

<INITIAL>scaling			{ nchars+=yyleng; BEGIN(GETSCALING); return _SCALING; }
<GETSCALING>(constrained|CONSTRAINED)	{ nchars+=yyleng; BEGIN(INITIAL); return _CONSTRAINED; }
<GETSCALING>(unconstrained|UNCONSTRAINED) { nchars+=yyleng; BEGIN(INITIAL); return _UNCONSTRAINED; }

<INITIAL>shading			{ nchars+=yyleng; BEGIN(GETSHADING); return _SHADING; }
<GETSHADING>(xyz|XYZ)			{ nchars+=yyleng; BEGIN(INITIAL); return _XYZ; }
<GETSHADING>(xy|XY)			{ nchars+=yyleng; BEGIN(INITIAL); return _XY; }
<GETSHADING>(z|Z)			{ nchars+=yyleng; BEGIN(INITIAL); return _Z; }
<GETSHADING>(zgrayscale|ZGRAYSCALE)	{ nchars+=yyleng; BEGIN(INITIAL); return _ZGRAYSCALE; }
<GETSHADING>(zhue|ZHUE)			{ nchars+=yyleng; BEGIN(INITIAL); return _ZHUE; }
<GETSHADING>(none|NONE)			{ nchars+=yyleng; BEGIN(INITIAL); return _NONE; }

<INITIAL>font				{ nchars+=yyleng; BEGIN(GETFONT); return _FONT; }
<INITIAL>axesfont			{ nchars+=yyleng; BEGIN(GETFONT); return _AXESFONT; }
<INITIAL>labelfont			{ nchars+=yyleng; BEGIN(GETFONT); return _LABELFONT; }
<INITIAL>titlefont			{ nchars+=yyleng; BEGIN(GETFONT); return _TITLEFONT; }

<GETFONT>(times|TIMES)			{ nchars+=yyleng; BEGIN(INITIAL); return _TIMES; }
<GETFONT>(courier|COURIER)		{ nchars+=yyleng; BEGIN(INITIAL); return _COURIER; }
<GETFONT>(helvetica|HELVETICA)		{ nchars+=yyleng; BEGIN(INITIAL); return _HELVETICA; }
<GETFONT>(symbol|SYMBOL)		{ nchars+=yyleng; BEGIN(INITIAL); return _SYMBOL; }

<INITIAL>(roman|ROMAN)			{ nchars+=yyleng; return _ROMAN; }
<INITIAL>(bold|BOLD)			{ nchars+=yyleng; return _BOLD; }
<INITIAL>(italic|ITALIC)		{ nchars+=yyleng; return _ITALIC; }
<INITIAL>(bolditalic|BOLDITALIC)	{ nchars+=yyleng; return _BOLDITALIC; }
<INITIAL>(oblique|OBLIQUE)		{ nchars+=yyleng; return _OBLIQUE; }
<INITIAL>(boldoblique|BOLDOBLIQUE)	{ nchars+=yyleng; return _BOLDOBLIQUE; }

<INITIAL>gridstyle			{ nchars+=yyleng; BEGIN(GETGRIDSTYLE); return _GRIDSTYLE; }
<GETGRIDSTYLE>(triangular|TRIANGULAR)	{ nchars+=yyleng; BEGIN(INITIAL); return _TRIANGULAR_GRIDLINES; }
<GETGRIDSTYLE>(rectangular|RECTANGULAR|xy_gridlines|XY_GRIDLINES) { nchars+=yyleng; BEGIN(INITIAL); return _XY_GRIDLINES; }
<GETGRIDSTYLE>(x_gridlines|X_GRIDLINES) { nchars+=yyleng; BEGIN(INITIAL); return _X_GRIDLINES; }
<GETGRIDSTYLE>(y_gridlines|Y_GRIDLINES) { nchars+=yyleng; BEGIN(INITIAL); return _Y_GRIDLINES; }

<INITIAL>coords				{ nchars+=yyleng; BEGIN(GETCOORDS); return _COORDS; }
<GETCOORDS>(cartesian|CARTESIAN)	{ nchars+=yyleng; BEGIN(INITIAL); return _CARTESIAN; }
<GETCOORDS>(cylindrical|CYLINDRICAL)	{ nchars+=yyleng; BEGIN(INITIAL); return _CYLINDRICAL; }
<GETCOORDS>(z_cylindrical|Z_CYLINDRICAL) { nchars+=yyleng; BEGIN(INITIAL); return _Z_CYLINDRICAL; }
<GETCOORDS>(spherical|SPHERICAL)	{ nchars+=yyleng; BEGIN(INITIAL); return _SPHERICAL; }

<INITIAL>align				{ nchars+=yyleng; BEGIN(GETALIGN); return _ALIGN; }
<GETALIGN>(left|LEFT)			{ nchars+=yyleng; BEGIN(INITIAL); return _LEFT; }
<GETALIGN>(right|RIGHT)			{ nchars+=yyleng; BEGIN(INITIAL); return _RIGHT; }
<GETALIGN>(above|ABOVE)			{ nchars+=yyleng; BEGIN(INITIAL); return _ABOVE; }
<GETALIGN>(below|BELOW)			{ nchars+=yyleng; BEGIN(INITIAL); return _BELOW; }


{NUM}\.+	{	/* number followed by a range spec '..' */
			char *p = yytext+yyleng-1;
			int i = 0;
			while (*p--=='.') i++;	/* count the number of dots */
			yyless(yyleng-i);	/* save the dots for later */
			yylval.str = copy_string(yytext);
			nchars+=yyleng;
			return _NUMBER;
		}

{NUM}		{ nchars+=yyleng; yylval.str = copy_string(yytext); return _NUMBER; }
Pi|E		{ nchars+=yyleng; yylval.str = copy_string(yytext); return _NUMBER; }
{ID}		{ nchars+=yyleng; yylval.str = copy_string(yytext); return _IDENT; }
{STR}		{ 	nchars+=yyleng;
			yytext[yyleng-1]='\0';		    /* strip quote */
			yylval.str = copy_string(1+yytext); /* strip quote */
			return _STRING;
		}

.		{ nchars+=yyleng; return yytext[0];	/* anything else */ }

%%

void show_parse_error(void)
{
	int i;

	for (i=0; i<nchars-yyleng+(int)strlen(PROMPT1); i++)
		putchar(' ');
	printf("^-- You goofed!\n");

	return;
}

/* This routine is not perfect, but we will let it go for now.
   The problem with it is that the opened files are not closed,
   therefore with repeated use we will run out of file descriptors.
   Yet another problem is that if the input file contains:
      plot something1
      read another file
      plot somethig2
   then something2 is not plotted because control does not return
   to a file once it's tranfered elsewhere.

   To see how to fix this, look up the string MAX_INCLUDE_DEPTH the
   flex man page.
*/
void read_from_file(char *filename)
{
	FILE *fp;

	if ((fp=fopen(filename, "r"))==NULL) {
		printf("\terror: unable to open file %s for reading\n",
				filename);
		return;
	}

	yyin = fp;
	yy_delete_buffer(YY_CURRENT_BUFFER);
	yy_switch_to_buffer(yy_create_buffer(yyin, YY_BUF_SIZE));
	BEGIN(INITIAL);
}

void read_from_string(char *line)
{
	yy_delete_buffer(YY_CURRENT_BUFFER);
	BEGIN(INITIAL);
	yy_scan_string(line);
}
