#include <stdlib.h>
#include <forms.h>
#include "formdefs.h"
#include "dynagraph.h"

extern GLfloat		*bgColor, *fgColor;
extern GLfloat		whiteColor[], blackColor[];
extern FD_form		*Form;

static void file_menu_cb(FL_OBJECT *obj, long arg);
static void style_menu_cb(FL_OBJECT *obj, long arg);
static void shading_menu_cb(FL_OBJECT *obj, long arg);
static void view_menu_cb(FL_OBJECT *obj, long arg);
static void options_menu_cb(FL_OBJECT *obj, long arg);

/* The menu definitions in this file are done in a much longer way than
   those described in the xforms manual.  The lengthier method here
   provides more flexibility in adding/altering menus.  

   Items on menus and their cascading sub-menus are listted in a
   enum list first.  (We have prefixed the name of each enum entry
   with "pup" to avoid possible conflict with definition of constants
   elsewhere.)  The menu entries will be created accroding to the order
   prescribed in these enum lists.  This is the only place where the
   order matters.  The actual definitions of the entries can be described
   in any order.  This implies that to switch the order of two entries in
   the menu, it is sufficient to switch the entries in the corresponding
   enum list; no other changes are necessary.  Similarly, items may
   be added or removed from a menu by modifying the enum list.  Other
   items are not affected.
*/


enum FileMenuList {
	pupBeginSAVE,
		pupJPEG,
		pupPNG,
		pupTIFF,
		pupPPM6,
		pupPPM3,
		pupPOSTSCRIPT,
		pupCOLORPOSTSCRIPT,
		pupVECTORPOSTSCRIPT,
	pupEndSAVE,

	pupCLOSE,
	pupEXIT,
	pupEndFileMenu
};

enum StyleMenuList {
	pupPATCHNOGRID,
	pupPATCH,
	pupPATCHCONTOUR,
	pupCONTOURS,
	pupHIDDEN,
	pupWIREFRAME,
	pupPOINT,

	pupBeginTHICKNESS,	/* cascade */
		pupTHICKNESS1,
		pupTHICKNESS2,
		pupTHICKNESS3,
		pupTHICKNESS4,
		pupTHICKNESS5,
		pupTHICKNESS6,
		pupTHICKNESS7,
		pupTHICKNESS8,
		pupTHICKNESS9,
		pupTHICKNESS10,
	pupEndTHICKNESS,

	pupBeginPOINTSIZE,	/* cascade */
		pupPOINTSIZE1,
		pupPOINTSIZE2,
		pupPOINTSIZE3,
		pupPOINTSIZE4,
		pupPOINTSIZE5,
		pupPOINTSIZE6,
		pupPOINTSIZE7,
		pupPOINTSIZE8,
		pupPOINTSIZE9,
		pupPOINTSIZE10,
	pupEndPOINTSIZE,

	pupBeginGRIDSTYLE,	/* cascade */
		pupXYGRIDLINES,
		pupXGRIDLINES,
		pupYGRIDLINES,
		pupTRIANGULARGRIDLINES,
	pupEndGRIDSTYLE,

	pupEndStyleMenu
};

enum ShadingMenuList {
	pupXYZ,
	pupXY,
	pupZ,
	pupZGRAYSCALE,
	pupZHUE,
	pupNOCOLOR,
	pupEndShadingMenu
};

enum ViewMenuList {
	pupPosX,
	pupPosY,
	pupPosZ,
	pupNegX,
	pupNegY,
	pupNegZ,
	pupDefaultView,
	pupEndViewMenu
};

enum OptionsMenuList {
	pupWhiteBg,
	pupBlackBg,
	pupSmoothShading,
	pupAntialias,
	pupEndOptionsMenu
};

static FL_PUP_ENTRY FileMenuDef[1+pupEndFileMenu];
static FL_PUP_ENTRY StyleMenuDef[1+pupEndStyleMenu];
static FL_PUP_ENTRY ShadingMenuDef[1+pupEndShadingMenu];
static FL_PUP_ENTRY ViewMenuDef[1+pupEndViewMenu];
static FL_PUP_ENTRY OptionsMenuDef[1+pupEndOptionsMenu];


void create_menus(void)
{
	FileMenuDef[pupBeginSAVE].text = "/_Save image to file";
	FileMenuDef[pupPOSTSCRIPT].text = "Postscript (gray)";
	FileMenuDef[pupPOSTSCRIPT].callback = NULL;
	FileMenuDef[pupPOSTSCRIPT].shortcut = "";
	FileMenuDef[pupPOSTSCRIPT].mode = FL_PUP_NONE;
	FileMenuDef[pupCOLORPOSTSCRIPT].text = "_Postscript (color)";
	FileMenuDef[pupCOLORPOSTSCRIPT].callback = NULL;
	FileMenuDef[pupCOLORPOSTSCRIPT].shortcut = "";
	FileMenuDef[pupCOLORPOSTSCRIPT].mode = FL_PUP_NONE;
	FileMenuDef[pupVECTORPOSTSCRIPT].text = "Postscript (color, vector)";
	FileMenuDef[pupVECTORPOSTSCRIPT].callback = NULL;
	FileMenuDef[pupVECTORPOSTSCRIPT].shortcut = "";
	FileMenuDef[pupVECTORPOSTSCRIPT].mode = FL_PUP_NONE;
	FileMenuDef[pupPNG].text = "PNG";
	FileMenuDef[pupPNG].callback = NULL;
	FileMenuDef[pupPNG].shortcut = "";
	FileMenuDef[pupPNG].mode = FL_PUP_NONE;
	FileMenuDef[pupPPM6].text = "PPM (binary)";
	FileMenuDef[pupPPM6].callback = NULL;
	FileMenuDef[pupPPM6].shortcut = "";
	FileMenuDef[pupPPM6].mode = FL_PUP_NONE;
	FileMenuDef[pupPPM3].text = "PPM (ascii)";
	FileMenuDef[pupPPM3].callback = NULL;
	FileMenuDef[pupPPM3].shortcut = "";
	FileMenuDef[pupPPM3].mode = FL_PUP_NONE;
	FileMenuDef[pupJPEG].text = "JPEG";
	FileMenuDef[pupJPEG].callback = NULL;
	FileMenuDef[pupJPEG].shortcut = "";
	FileMenuDef[pupJPEG].mode = FL_PUP_NONE;
	FileMenuDef[pupTIFF].text = "TIFF";
	FileMenuDef[pupTIFF].callback = NULL;
	FileMenuDef[pupTIFF].shortcut = "";
	FileMenuDef[pupTIFF].mode = FL_PUP_NONE;
	FileMenuDef[pupEndSAVE].text = NULL;
	FileMenuDef[pupCLOSE].text = "Close window";
	FileMenuDef[pupCLOSE].callback = NULL;
	FileMenuDef[pupCLOSE].shortcut = "";
	FileMenuDef[pupCLOSE].mode = FL_PUP_NONE;
	FileMenuDef[pupEXIT].text = "Exit Dynagraph";
	FileMenuDef[pupEXIT].callback = NULL;
	FileMenuDef[pupEXIT].shortcut = "";
	FileMenuDef[pupEXIT].mode = FL_PUP_NONE;
	FileMenuDef[pupEndFileMenu].text = NULL;

	StyleMenuDef[pupPATCH].text = "Surface + Grid";
	StyleMenuDef[pupPATCH].callback = NULL;
	StyleMenuDef[pupPATCH].shortcut = "";
	StyleMenuDef[pupPATCH].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupPATCHNOGRID].text = "Surface  (default)";
	StyleMenuDef[pupPATCHNOGRID].callback = NULL;
	StyleMenuDef[pupPATCHNOGRID].shortcut = "";
	StyleMenuDef[pupPATCHNOGRID].mode = FL_PUP_RADIO|FL_PUP_CHECK;
	StyleMenuDef[pupPATCHCONTOUR].text = "Surface + Contours";
	StyleMenuDef[pupPATCHCONTOUR].callback = NULL;
	StyleMenuDef[pupPATCHCONTOUR].shortcut = "";
	StyleMenuDef[pupPATCHCONTOUR].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupHIDDEN].text = "Hidden Line";
	StyleMenuDef[pupHIDDEN].callback = NULL;
	StyleMenuDef[pupHIDDEN].shortcut = "";
	StyleMenuDef[pupHIDDEN].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupCONTOURS].text = "Contours";
	StyleMenuDef[pupCONTOURS].callback = NULL;
	StyleMenuDef[pupCONTOURS].shortcut = "";
	StyleMenuDef[pupCONTOURS].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupWIREFRAME].text = "Line";
	StyleMenuDef[pupWIREFRAME].callback = NULL;
	StyleMenuDef[pupWIREFRAME].shortcut = "";
	StyleMenuDef[pupWIREFRAME].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupPOINT].text = "_Point";
	StyleMenuDef[pupPOINT].callback = NULL;
	StyleMenuDef[pupPOINT].shortcut = "";
	StyleMenuDef[pupPOINT].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupBeginTHICKNESS].text = "/Line Width";
	StyleMenuDef[pupTHICKNESS1].text = "1 pixel (default)";
	StyleMenuDef[pupTHICKNESS1].callback = NULL;
	StyleMenuDef[pupTHICKNESS1].shortcut = "";
	StyleMenuDef[pupTHICKNESS1].mode = FL_PUP_RADIO|FL_PUP_CHECK;
	StyleMenuDef[pupTHICKNESS2].text = "2 pixels";
	StyleMenuDef[pupTHICKNESS2].callback = NULL;
	StyleMenuDef[pupTHICKNESS2].shortcut = "";
	StyleMenuDef[pupTHICKNESS2].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupTHICKNESS3].text = "3 pixels";
	StyleMenuDef[pupTHICKNESS3].callback = NULL;
	StyleMenuDef[pupTHICKNESS3].shortcut = "";
	StyleMenuDef[pupTHICKNESS3].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupTHICKNESS4].text = "4 pixels";
	StyleMenuDef[pupTHICKNESS4].callback = NULL;
	StyleMenuDef[pupTHICKNESS4].shortcut = "";
	StyleMenuDef[pupTHICKNESS4].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupTHICKNESS5].text = "5 pixels";
	StyleMenuDef[pupTHICKNESS5].callback = NULL;
	StyleMenuDef[pupTHICKNESS5].shortcut = "";
	StyleMenuDef[pupTHICKNESS5].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupTHICKNESS6].text = "6 pixels";
	StyleMenuDef[pupTHICKNESS6].callback = NULL;
	StyleMenuDef[pupTHICKNESS6].shortcut = "";
	StyleMenuDef[pupTHICKNESS6].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupTHICKNESS7].text = "7 pixels";
	StyleMenuDef[pupTHICKNESS7].callback = NULL;
	StyleMenuDef[pupTHICKNESS7].shortcut = "";
	StyleMenuDef[pupTHICKNESS7].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupTHICKNESS8].text = "8 pixels";
	StyleMenuDef[pupTHICKNESS8].callback = NULL;
	StyleMenuDef[pupTHICKNESS8].shortcut = "";
	StyleMenuDef[pupTHICKNESS8].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupTHICKNESS9].text = "9 pixels";
	StyleMenuDef[pupTHICKNESS9].callback = NULL;
	StyleMenuDef[pupTHICKNESS9].shortcut = "";
	StyleMenuDef[pupTHICKNESS9].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupTHICKNESS10].text = "10 pixels";
	StyleMenuDef[pupTHICKNESS10].callback = NULL;
	StyleMenuDef[pupTHICKNESS10].shortcut = "";
	StyleMenuDef[pupTHICKNESS10].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupEndTHICKNESS].text = NULL;
	StyleMenuDef[pupBeginPOINTSIZE].text = "/Point Size";
	StyleMenuDef[pupPOINTSIZE1].text = "1 pixel";
	StyleMenuDef[pupPOINTSIZE1].callback = NULL;
	StyleMenuDef[pupPOINTSIZE1].shortcut = "";
	StyleMenuDef[pupPOINTSIZE1].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupPOINTSIZE2].text = "2 pixels";
	StyleMenuDef[pupPOINTSIZE2].callback = NULL;
	StyleMenuDef[pupPOINTSIZE2].shortcut = "";
	StyleMenuDef[pupPOINTSIZE2].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupPOINTSIZE3].text = "3 pixels (default)";
	StyleMenuDef[pupPOINTSIZE3].callback = NULL;
	StyleMenuDef[pupPOINTSIZE3].shortcut = "";
	StyleMenuDef[pupPOINTSIZE3].mode = FL_PUP_RADIO|FL_PUP_CHECK;
	StyleMenuDef[pupPOINTSIZE4].text = "4 pixels";
	StyleMenuDef[pupPOINTSIZE4].callback = NULL;
	StyleMenuDef[pupPOINTSIZE4].shortcut = "";
	StyleMenuDef[pupPOINTSIZE4].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupPOINTSIZE5].text = "5 pixels";
	StyleMenuDef[pupPOINTSIZE5].callback = NULL;
	StyleMenuDef[pupPOINTSIZE5].shortcut = "";
	StyleMenuDef[pupPOINTSIZE5].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupPOINTSIZE6].text = "6 pixels";
	StyleMenuDef[pupPOINTSIZE6].callback = NULL;
	StyleMenuDef[pupPOINTSIZE6].shortcut = "";
	StyleMenuDef[pupPOINTSIZE6].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupPOINTSIZE7].text = "7 pixels";
	StyleMenuDef[pupPOINTSIZE7].callback = NULL;
	StyleMenuDef[pupPOINTSIZE7].shortcut = "";
	StyleMenuDef[pupPOINTSIZE7].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupPOINTSIZE8].text = "8 pixels";
	StyleMenuDef[pupPOINTSIZE8].callback = NULL;
	StyleMenuDef[pupPOINTSIZE8].shortcut = "";
	StyleMenuDef[pupPOINTSIZE8].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupPOINTSIZE9].text = "9 pixels";
	StyleMenuDef[pupPOINTSIZE9].callback = NULL;
	StyleMenuDef[pupPOINTSIZE9].shortcut = "";
	StyleMenuDef[pupPOINTSIZE9].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupPOINTSIZE10].text = "10 pixels";
	StyleMenuDef[pupPOINTSIZE10].callback = NULL;
	StyleMenuDef[pupPOINTSIZE10].shortcut = "";
	StyleMenuDef[pupPOINTSIZE10].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupEndPOINTSIZE].text = NULL;
	StyleMenuDef[pupBeginGRIDSTYLE].text = "/Grid Style";
	StyleMenuDef[pupXYGRIDLINES].text = "Rectangular mesh";
	StyleMenuDef[pupXYGRIDLINES].callback = NULL;
	StyleMenuDef[pupXYGRIDLINES].shortcut = "";
	StyleMenuDef[pupXYGRIDLINES].mode = FL_PUP_RADIO|FL_PUP_CHECK;
	StyleMenuDef[pupXGRIDLINES].text = "Constant X lines";
	StyleMenuDef[pupXGRIDLINES].callback = NULL;
	StyleMenuDef[pupXGRIDLINES].shortcut = "";
	StyleMenuDef[pupXGRIDLINES].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupYGRIDLINES].text = "Constant Y lines";
	StyleMenuDef[pupYGRIDLINES].callback = NULL;
	StyleMenuDef[pupYGRIDLINES].shortcut = "";
	StyleMenuDef[pupYGRIDLINES].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupTRIANGULARGRIDLINES].text = "Triangular mesh";
	StyleMenuDef[pupTRIANGULARGRIDLINES].callback = NULL;
	StyleMenuDef[pupTRIANGULARGRIDLINES].shortcut = "";
	StyleMenuDef[pupTRIANGULARGRIDLINES].mode = FL_PUP_RADIO|FL_PUP_BOX;

	StyleMenuDef[pupEndGRIDSTYLE].text = NULL;
	StyleMenuDef[pupEndStyleMenu].text = NULL;

	ShadingMenuDef[pupXYZ].text = "XYZ";
	ShadingMenuDef[pupXYZ].callback = NULL;
	ShadingMenuDef[pupXYZ].shortcut = "";
	ShadingMenuDef[pupXYZ].mode = FL_PUP_RADIO|FL_PUP_CHECK;
	ShadingMenuDef[pupXY].text = "XY";
	ShadingMenuDef[pupXY].callback = NULL;
	ShadingMenuDef[pupXY].shortcut = "";
	ShadingMenuDef[pupXY].mode = FL_PUP_RADIO|FL_PUP_BOX;
	ShadingMenuDef[pupZ].text = "Z";
	ShadingMenuDef[pupZ].callback = NULL;
	ShadingMenuDef[pupZ].shortcut = "";
	ShadingMenuDef[pupZ].mode = FL_PUP_RADIO|FL_PUP_BOX;
	ShadingMenuDef[pupZGRAYSCALE].text = "Z Grayscale";
	ShadingMenuDef[pupZGRAYSCALE].callback = NULL;
	ShadingMenuDef[pupZGRAYSCALE].shortcut = "";
	ShadingMenuDef[pupZGRAYSCALE].mode = FL_PUP_RADIO|FL_PUP_BOX;
	ShadingMenuDef[pupZHUE].text = "Z HUE";
	ShadingMenuDef[pupZHUE].callback = NULL;
	ShadingMenuDef[pupZHUE].shortcut = "";
	ShadingMenuDef[pupZHUE].mode = FL_PUP_RADIO|FL_PUP_BOX;
	ShadingMenuDef[pupNOCOLOR].text = "White";
	ShadingMenuDef[pupNOCOLOR].callback = NULL;
	ShadingMenuDef[pupNOCOLOR].shortcut = "";
	ShadingMenuDef[pupNOCOLOR].mode = FL_PUP_RADIO|FL_PUP_BOX;
	ShadingMenuDef[pupEndShadingMenu].text = NULL;

	ViewMenuDef[pupPosX].text = "View from the +X direction";
	ViewMenuDef[pupPosX].callback = NULL;
	ViewMenuDef[pupPosX].shortcut = "";
	ViewMenuDef[pupPosX].mode = FL_PUP_NONE;
	ViewMenuDef[pupPosY].text = "View from the +Y direction";
	ViewMenuDef[pupPosY].callback = NULL;
	ViewMenuDef[pupPosY].shortcut = "";
	ViewMenuDef[pupPosY].mode = FL_PUP_NONE;
	ViewMenuDef[pupPosZ].text = "View from the +Z direction";
	ViewMenuDef[pupPosZ].callback = NULL;
	ViewMenuDef[pupPosZ].shortcut = "";
	ViewMenuDef[pupPosZ].mode = FL_PUP_NONE;
	ViewMenuDef[pupNegX].text = "View from the -X direction";
	ViewMenuDef[pupNegX].callback = NULL;
	ViewMenuDef[pupNegX].shortcut = "";
	ViewMenuDef[pupNegX].mode = FL_PUP_NONE;
	ViewMenuDef[pupNegY].text = "View from the -Y direction";
	ViewMenuDef[pupNegY].callback = NULL;
	ViewMenuDef[pupNegY].shortcut = "";
	ViewMenuDef[pupNegY].mode = FL_PUP_NONE;
	ViewMenuDef[pupNegZ].text = "View from the -Z direction";
	ViewMenuDef[pupNegZ].callback = NULL;
	ViewMenuDef[pupNegZ].shortcut = "";
	ViewMenuDef[pupNegZ].mode = FL_PUP_NONE;
	ViewMenuDef[pupDefaultView].text = "Default View";
	ViewMenuDef[pupDefaultView].callback = NULL;
	ViewMenuDef[pupDefaultView].shortcut = "";
	ViewMenuDef[pupDefaultView].mode = FL_PUP_NONE;
	ViewMenuDef[pupEndViewMenu].text = NULL;

	OptionsMenuDef[pupWhiteBg].text = "White Background";
	OptionsMenuDef[pupWhiteBg].callback = NULL;
	OptionsMenuDef[pupWhiteBg].shortcut = "";
	OptionsMenuDef[pupWhiteBg].mode = FL_PUP_CHECK;
	OptionsMenuDef[pupBlackBg].text = "_Black Background";
	OptionsMenuDef[pupBlackBg].callback = NULL;
	OptionsMenuDef[pupBlackBg].shortcut = "";
	OptionsMenuDef[pupBlackBg].mode = FL_PUP_BOX;

	OptionsMenuDef[pupSmoothShading].text = "_Smoothly shaded surfaces";
	OptionsMenuDef[pupSmoothShading].callback = NULL;
	OptionsMenuDef[pupSmoothShading].shortcut = "";
	OptionsMenuDef[pupSmoothShading].mode = FL_PUP_CHECK;

	OptionsMenuDef[pupAntialias].text = "Antialiased curves and points";
	OptionsMenuDef[pupAntialias].callback = NULL;
	OptionsMenuDef[pupAntialias].shortcut = "";
	OptionsMenuDef[pupAntialias].mode = FL_PUP_CHECK;
	OptionsMenuDef[pupEndOptionsMenu].text = NULL;

	fl_set_menu_entries(Form->file_menu, FileMenuDef);
	fl_set_object_callback(Form->file_menu, file_menu_cb, 0);

	fl_set_menu_entries(Form->style_menu, StyleMenuDef);
	fl_set_object_callback(Form->style_menu, style_menu_cb, 0);

	fl_set_menu_entries(Form->shading_menu, ShadingMenuDef);
	fl_set_object_callback(Form->shading_menu, shading_menu_cb, 0);

	fl_set_menu_entries(Form->view_menu, ViewMenuDef);
	fl_set_object_callback(Form->view_menu, view_menu_cb, 0);

	fl_set_menu_entries(Form->options_menu, OptionsMenuDef);
	fl_set_object_callback(Form->options_menu, options_menu_cb, 0);
}

/* returns 0 or one indicating the state of the antialias menu selecton */
int antialias(void)
{
	return fl_get_menu_item_mode(Form->options_menu, pupAntialias+1)
		& FL_PUP_CHECK;
}


static void file_menu_cb(FL_OBJECT *obj, long arg)
{
	FL_OBJECT *glCanvas = Form->glCanvas;
	int item = fl_get_menu(obj)-1;
	SavegraphData *sgd;

	sgd = malloc(sizeof(SavegraphData));
	sgd->file = NULL;
	sgd->plot3d = glCanvas->u_vdata;
	
	switch (item) {
		case pupJPEG:
			sgd->fmt = JPEG_FORMAT;
			save_graph_to_file(sgd);
			break;
		case pupTIFF:
			sgd->fmt = TIFF_FORMAT;
			save_graph_to_file(sgd);
			break;
		case pupPPM6:
			sgd->fmt = PPM6_FORMAT;
			save_graph_to_file(sgd);
			break;
		case pupPPM3:
			sgd->fmt = PPM3_FORMAT;
			save_graph_to_file(sgd);
			break;
		case pupPOSTSCRIPT:
			sgd->fmt = PS_FORMAT;
			save_graph_to_file(sgd);
			break;
		case pupCOLORPOSTSCRIPT:
			sgd->fmt = CPS_FORMAT;
			save_graph_to_file(sgd);
			break;
		case pupVECTORPOSTSCRIPT:
			sgd->fmt = VPS_FORMAT;
			save_graph_to_file(sgd);
			break;
		case pupPNG:
			sgd->fmt = PNG_FORMAT;
			save_graph_to_file(sgd);
			break;
		case pupCLOSE:
			remove_canvas_handlers();
			free_plotlist(glCanvas->u_vdata);
/* 			fl_delete_object(glCanvas); */
/* 			fl_free_object(glCanvas); */
			fl_hide_form(Form->form);
			fl_free_form(Form->form);
			free(Form);
			Form = NULL;
			break;
		case pupEXIT:
			call_exit();
	}
}

static void set_plot_style(unsigned int style)
{
	FL_OBJECT *glCanvas = Form->glCanvas;
	Plot3Dattribs *a;
	Plot3D *p, *q;

	p = glCanvas->u_vdata;
	a = NULL;
	for (q=p; q!=NULL; q=q->fp) {
		if (q->attr==a)
			continue;
		a = q->attr;
		if (!(a->style&locked))
			a->style = style;
	}
	show_form();
	create_displaylists(p);
/* 	install_plot(p); */
	display();
}

static void set_plot_thickness(unsigned int thickness)
{
	FL_OBJECT *glCanvas = Form->glCanvas;
	Plot3Dattribs *a;
	Plot3D *p, *q;

	p = glCanvas->u_vdata;
	a = NULL;
	for (q=p; q!=NULL; q=q->fp) {
		if (q->attr==a)
			continue;
		a = q->attr;
		if (!(a->thickness&locked))
			a->thickness = (1<<thickness);
	}
	show_form();
	create_displaylists(p);
/* 	install_plot(p); */
	display();
}

static void set_plot_pointsize(unsigned int pointsize)
{
	FL_OBJECT *glCanvas = Form->glCanvas;
	Plot3Dattribs *a;
	Plot3D *p, *q;

	p = glCanvas->u_vdata;
	a = NULL;
	for (q=p; q!=NULL; q=q->fp) {
		if (q->attr==a)
			continue;
		a = q->attr;
		if (!(a->pointsize&locked))
			a->pointsize = (1<<pointsize);
	}
	show_form();
	create_displaylists(p);
/* 	install_plot(p); */
	display();
}

static void set_plot_gridstyle(unsigned int gridstyle)
{
	FL_OBJECT *glCanvas = Form->glCanvas;
	Plot3Dattribs *a;
	Plot3D *p, *q;

	p = glCanvas->u_vdata;
	a = NULL;
	for (q=p; q!=NULL; q=q->fp) {
		if (q->attr==a)
			continue;
		a = q->attr;

		if (!(a->gridstyle&locked))
			a->gridstyle = gridstyle;
	}
	show_form();
	create_displaylists(p);
	display();
}

static void style_menu_cb(FL_OBJECT *obj, long arg)
{
	int item = fl_get_menu(obj)-1;

	switch(item) {

		case pupPATCH:
			set_plot_style(PATCH);
			break;
		case pupPATCHNOGRID:
			set_plot_style(PATCHNOGRID);
			break;
		case pupPATCHCONTOUR:
			set_plot_style(PATCHCONTOUR);
			break;
		case pupHIDDEN:
			set_plot_style(HIDDEN);
			break;
		case pupCONTOURS:
			set_plot_style(CONTOUR);
			break;
		case pupWIREFRAME:
			set_plot_style(WIREFRAME);
			break;
		case pupPOINT:
			set_plot_style(POINT);
			break;

		case pupTHICKNESS1:
		case pupTHICKNESS2:
		case pupTHICKNESS3:
		case pupTHICKNESS4:
		case pupTHICKNESS5:
		case pupTHICKNESS6:
		case pupTHICKNESS7:
		case pupTHICKNESS8:
		case pupTHICKNESS9:
		case pupTHICKNESS10:
			set_plot_thickness(item - pupBeginTHICKNESS);
			break;

		case pupPOINTSIZE1:
		case pupPOINTSIZE2:
		case pupPOINTSIZE3:
		case pupPOINTSIZE4:
		case pupPOINTSIZE5:
		case pupPOINTSIZE6:
		case pupPOINTSIZE7:
		case pupPOINTSIZE8:
		case pupPOINTSIZE9:
		case pupPOINTSIZE10:
			set_plot_pointsize(item - pupBeginPOINTSIZE);
			break;

		case pupXYGRIDLINES:
			set_plot_gridstyle(XY_GRIDLINES);
			break;

		case pupXGRIDLINES:
			set_plot_gridstyle(X_GRIDLINES);
			break;

		case pupYGRIDLINES:
			set_plot_gridstyle(Y_GRIDLINES);
			break;

		case pupTRIANGULARGRIDLINES:
			set_plot_gridstyle(TRIANGULAR_GRIDLINES);
			break;
	}
}

static void shading_menu_cb(FL_OBJECT *obj, long arg)
{
	FL_OBJECT *glCanvas = Form->glCanvas;
	int item = fl_get_menu(obj)-1;
	unsigned int shading;
	Plot3Dattribs *a;
	Plot3D *p, *q;

	switch(item) {
		case pupXYZ:
			shading = XYZSHADING;
			break;
		case pupXY:
			shading = XYSHADING;
			break;
		case pupZ:
			shading = ZSHADING;
			break;
		case pupZGRAYSCALE:
			shading = ZGRAYSCALE;
			break;
		case pupZHUE:
			shading = ZHUE;
			break;
		case pupNOCOLOR:
			shading = NOSHADING;
			break;
		default:
			return;
	}

	p = glCanvas->u_vdata;
	a = NULL;
	for (q=p; q!=NULL; q=q->fp) {
		if (q->attr==a)
			continue;
		a = q->attr;

		if (!(a->shading&locked))
			a->shading = shading;
	}
	show_form();
	create_displaylists(p);
/* 	install_plot(p); */
	display();
}

static void view_menu_cb(FL_OBJECT *obj, long arg)
{
	int item = fl_get_menu(obj)-1;

	glLoadIdentity();

	switch (item) {
		case pupPosX:		/* view from +x */
			glRotatef(-120, 1, 1, 1);
			break;
		case pupPosY:		/* view from +y */
			glRotatef(+180, 0, 1, 1);
			break;
		case pupPosZ:		/* view from +z */
			/* default orientation */
			break;
		case pupNegX:		/* view from -x */
			glRotatef(120, -1, 1, 1);
			break;
		case pupNegY:		/* view from -y */
			glRotatef(-90, 1, 0, 0);
			break;
		case pupNegZ:		/* view from -z */
			glRotatef(180, 0, 1, 0);
			break;
		case pupDefaultView:		/* standard view */
			glRotatef(-(90+DEFAULT_VIEW_LONGITUDE), 0, 0, 1);
			glRotatef(DEFAULT_VIEW_COLATITUDE, 1, -1, 0);
			break;
		default:
			return;
	}

	/* call the zoom dial callback to scale the object accroding to
	   the dial setting.  zoomdial_cb() repaints the screen, therefore
	   we do not need to do an explicit repaint here. */
	zoomdial_cb(Form->zoomdial, 1);
}

static void options_menu_cb(FL_OBJECT *obj, long arg)
{
	FL_OBJECT *glCanvas = Form->glCanvas;
	int item = fl_get_menu(obj)-1;
	Plot3D *p;

	p = glCanvas->u_vdata;

	switch (item) {
		case pupWhiteBg:
			bgColor = whiteColor;
			fgColor = blackColor;
			glClearColor(bgColor[0], bgColor[1], bgColor[2], bgColor[3]);
			OptionsMenuDef[pupWhiteBg].mode = FL_PUP_CHECK;
			OptionsMenuDef[pupBlackBg].mode = FL_PUP_BOX;
			fl_set_menu_entries(obj, OptionsMenuDef);
			/* rebuild, because font colors change */
			show_form();
			create_displaylists(p);
			break;
		case pupBlackBg:
			bgColor = blackColor;
			fgColor = whiteColor;
			glClearColor(bgColor[0], bgColor[1], bgColor[2], bgColor[3]);
			OptionsMenuDef[pupWhiteBg].mode = FL_PUP_BOX;
			OptionsMenuDef[pupBlackBg].mode = FL_PUP_CHECK;
			fl_set_menu_entries(obj, OptionsMenuDef);
			/* rebuild, because font characters need to change */
			show_form();
			create_displaylists(p);
			break;
		case pupSmoothShading:
			OptionsMenuDef[pupSmoothShading].mode = 
				fl_get_menu_item_mode(obj, pupSmoothShading+1);
			if (OptionsMenuDef[pupSmoothShading].mode &FL_PUP_CHECK)
				glShadeModel(GL_SMOOTH);
			else
				glShadeModel(GL_FLAT);
			break;
		case pupAntialias:
			OptionsMenuDef[pupAntialias].mode = 
				fl_get_menu_item_mode(obj, pupAntialias+1);
			show_form();
			create_displaylists(p);
			break;
		default:
				return;
	}

	display();
}

static int exists_free_shading(void)
{
	FL_OBJECT *glCanvas = Form->glCanvas;
	Plot3D *p, *q;
	Plot3Dattribs *a;

	p = glCanvas->u_vdata;
	a = NULL;
	for (q=p; q!=NULL; q=q->fp) {
		if (q->attr==a)
			continue;
		a = q->attr;

		if (!(a->shading&locked))
			return 1;
	}
	return 0;
}

static int exists_free_style(void)
{
	FL_OBJECT *glCanvas = Form->glCanvas;
	Plot3D *p, *q;
	Plot3Dattribs *a;

	p = glCanvas->u_vdata;
	a = NULL;
	for (q=p; q!=NULL; q=q->fp) {
		if (q->attr==a)
			continue;
		a = q->attr;

		if (!(a->style&locked))
			return 1;
	}
	return 0;
}

static int exists_free_gridstyle(void)
{
	FL_OBJECT *glCanvas = Form->glCanvas;
	Plot3D *p, *q;
	Plot3Dattribs *a;

	p = glCanvas->u_vdata;
	a = NULL;
	for (q=p; q!=NULL; q=q->fp) {
		if (q->attr==a)
			continue;
		a = q->attr;

		if (!(a->gridstyle&locked))
			return 1;
	}
	return 0;
}

static int exists_free_thickness(void)
{
	FL_OBJECT *glCanvas = Form->glCanvas;
	Plot3D *p, *q;
	Plot3Dattribs *a;

	p = glCanvas->u_vdata;
	a = NULL;
	for (q=p; q!=NULL; q=q->fp) {
		if (q->attr==a)
			continue;
		a = q->attr;

		if (!(a->thickness&locked))
			return 1;
	}
	return 0;
}

static int exists_free_pointsize(void)
{
	FL_OBJECT *glCanvas = Form->glCanvas;
	Plot3D *p, *q;
	Plot3Dattribs *a;

	p = glCanvas->u_vdata;
	a = NULL;
	for (q=p; q!=NULL; q=q->fp) {
		if (q->attr==a)
			continue;
		a = q->attr;

		if (!(a->pointsize&locked))
			return 1;
	}
	return 0;
}

/* This function is called following the reading of a new input data.
   It sets the status of the menu items according to the received data.
*/
void set_menu_options(void)
{
	if (exists_free_shading()) {
		ShadingMenuDef[pupXYZ].mode	= FL_PUP_RADIO|FL_PUP_CHECK;
		ShadingMenuDef[pupXY].mode	= FL_PUP_RADIO|FL_PUP_BOX;
		ShadingMenuDef[pupZ].mode	= FL_PUP_RADIO|FL_PUP_BOX;
		ShadingMenuDef[pupZGRAYSCALE].mode = FL_PUP_RADIO|FL_PUP_BOX;
		ShadingMenuDef[pupZHUE].mode	= FL_PUP_RADIO|FL_PUP_BOX;
		ShadingMenuDef[pupNOCOLOR].mode	= FL_PUP_RADIO|FL_PUP_BOX;
	} else {
		ShadingMenuDef[pupXYZ].mode	= FL_PUP_GRAY;
		ShadingMenuDef[pupXY].mode	= FL_PUP_GRAY;
		ShadingMenuDef[pupZ].mode	= FL_PUP_GRAY;
		ShadingMenuDef[pupZGRAYSCALE].mode = FL_PUP_GRAY;
		ShadingMenuDef[pupZHUE].mode	= FL_PUP_GRAY;
		ShadingMenuDef[pupNOCOLOR].mode	= FL_PUP_GRAY;
	}

	if (exists_free_style()) {
		StyleMenuDef[pupPATCHNOGRID].mode = FL_PUP_RADIO|FL_PUP_CHECK;
		StyleMenuDef[pupPATCH].mode = FL_PUP_RADIO|FL_PUP_BOX;
		StyleMenuDef[pupHIDDEN].mode = FL_PUP_RADIO|FL_PUP_BOX;
		StyleMenuDef[pupWIREFRAME].mode = FL_PUP_RADIO|FL_PUP_BOX;
		StyleMenuDef[pupPOINT].mode = FL_PUP_RADIO|FL_PUP_BOX;
		StyleMenuDef[pupPATCHCONTOUR].mode = FL_PUP_RADIO|FL_PUP_BOX;
		StyleMenuDef[pupCONTOURS].mode = FL_PUP_RADIO|FL_PUP_BOX;
	} else {
		StyleMenuDef[pupPATCHNOGRID].mode	= FL_PUP_GRAY;
		StyleMenuDef[pupPATCH].mode		= FL_PUP_GRAY;
		StyleMenuDef[pupHIDDEN].mode		= FL_PUP_GRAY;
		StyleMenuDef[pupWIREFRAME].mode		= FL_PUP_GRAY;
		StyleMenuDef[pupPOINT].mode		= FL_PUP_GRAY;
		StyleMenuDef[pupPATCHCONTOUR].mode	= FL_PUP_GRAY;
		StyleMenuDef[pupCONTOURS].mode		= FL_PUP_GRAY;
	}

	StyleMenuDef[pupTHICKNESS1].mode = FL_PUP_RADIO|FL_PUP_CHECK;
	StyleMenuDef[pupTHICKNESS2].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupTHICKNESS3].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupTHICKNESS4].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupTHICKNESS5].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupTHICKNESS6].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupTHICKNESS7].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupTHICKNESS8].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupTHICKNESS9].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupTHICKNESS10].mode = FL_PUP_RADIO|FL_PUP_BOX;
	if (exists_free_thickness())
		StyleMenuDef[pupBeginTHICKNESS].mode = FL_PUP_NONE;
	else
		StyleMenuDef[pupBeginTHICKNESS].mode = FL_PUP_GRAY;

	StyleMenuDef[pupPOINTSIZE1].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupPOINTSIZE2].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupPOINTSIZE3].mode = FL_PUP_RADIO|FL_PUP_CHECK;
	StyleMenuDef[pupPOINTSIZE4].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupPOINTSIZE5].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupPOINTSIZE6].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupPOINTSIZE7].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupPOINTSIZE8].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupPOINTSIZE9].mode = FL_PUP_RADIO|FL_PUP_BOX;
	StyleMenuDef[pupPOINTSIZE10].mode = FL_PUP_RADIO|FL_PUP_BOX;
	if (exists_free_pointsize())
		StyleMenuDef[pupBeginPOINTSIZE].mode = FL_PUP_NONE;
	else
		StyleMenuDef[pupBeginPOINTSIZE].mode = FL_PUP_GRAY;

	if (exists_free_gridstyle()) {
		StyleMenuDef[pupXYGRIDLINES].mode = FL_PUP_RADIO|FL_PUP_CHECK;
		StyleMenuDef[pupXGRIDLINES].mode = FL_PUP_RADIO|FL_PUP_BOX;
		StyleMenuDef[pupYGRIDLINES].mode = FL_PUP_RADIO|FL_PUP_BOX;
		StyleMenuDef[pupTRIANGULARGRIDLINES].mode = FL_PUP_RADIO|FL_PUP_BOX;
	} else {
		StyleMenuDef[pupXYGRIDLINES].mode	= FL_PUP_GRAY;
		StyleMenuDef[pupXGRIDLINES].mode	= FL_PUP_GRAY;
		StyleMenuDef[pupYGRIDLINES].mode	= FL_PUP_GRAY;
		StyleMenuDef[pupTRIANGULARGRIDLINES].mode	= FL_PUP_GRAY;
	}

	fl_set_menu_entries(Form->style_menu, StyleMenuDef);
	fl_set_menu_entries(Form->shading_menu, ShadingMenuDef);
}
