#include <stdlib.h>
#include <float.h>	/* for DBL_MAX */
#include <string.h>	/* for strtok() */
#include "dynagraph.h"
#include "parse.h"

#define INHERIT_2ND_NARG(a, b)		if (!(a&locked)) a = b

#define INHERIT_2ND_PARG(a, b)		if (a==NULL)       a = b

#define IMPOSE_2ND_NARG(a, b)		if (b&locked) a = b

#define IMPOSE_2ND_SARG(a, b)		\
	if (b != NULL) {		\
		if (a != NULL)		\
			free(a);	\
		a = copy_string(b);	\
	}

#define INHERIT_2ND_SARG(a, b)		\
	if (a==NULL)			\
		a = copy_string(b);


/* Concatenates two Plot3D structures. */
static void merge2(Plot3D *p, Plot3D *q)
{
	Plot3Dattribs *pa, *qa;
	Plot3D *r;

	pa = p->attr;
	qa = q->attr;

	INHERIT_2ND_PARG(pa->xlabel, qa->xlabel);
	INHERIT_2ND_PARG(pa->ylabel, qa->ylabel);
	INHERIT_2ND_PARG(pa->zlabel, qa->zlabel);

	INHERIT_2ND_PARG(pa->title, qa->title);
	INHERIT_2ND_NARG(pa->titlefont, qa->titlefont);
	INHERIT_2ND_NARG(pa->titlewidth, qa->titlewidth);

	INHERIT_2ND_NARG(pa->axes, qa->axes);

	INHERIT_2ND_NARG(pa->font, qa->font);
	INHERIT_2ND_NARG(pa->axesfont, qa->axesfont);
	INHERIT_2ND_NARG(pa->labelfont, qa->labelfont);

	INHERIT_2ND_NARG(pa->xticks, qa->xticks);
	INHERIT_2ND_NARG(pa->yticks, qa->yticks);
	INHERIT_2ND_NARG(pa->zticks, qa->zticks);

	pa->nplots = qa->nplots = pa->nplots + qa->nplots;

	/* find end of p */
	for (r=p; r->fp!=NULL; r=r->fp)
		;

	r->fp = q;
	q->bp = r;
}

static Plot3D *merge_plot3d_structures(char *dispargs)
{
	char *id, *tmp;
	Plot3D *p, *q;

	p = NULL;

	tmp = dispargs;
	while ((id=strtok(tmp, ","))!=NULL) {
		if ((q = getplot(id))!=NULL) {
			q = copy_plotlist(q);	/* make a copy */
			if (p==NULL)
				p = q;
			else
				merge2(p, q);
		} else
			printf("\twarning: undefined plot %s\n", id);
		tmp = NULL;
	}

	return p;
}


Plot3D *make_merged_plot(char *dispargs, Plot3Dattribs *a)
{
	Plot3D *p, *r;
	Plot3Dattribs *saveattr;
	double d;

	p = merge_plot3d_structures(dispargs);

	if (p==NULL)
		return p;

	saveattr = NULL;
	for (r=p; r!=NULL; r=r->fp) {

		if (r->attr==saveattr)
			continue;

		saveattr = r->attr;

		INHERIT_2ND_SARG(r->attr->color, a->color);

		IMPOSE_2ND_SARG(r->attr->xlabel, a->xlabel);
		IMPOSE_2ND_SARG(r->attr->ylabel, a->ylabel);
		IMPOSE_2ND_SARG(r->attr->zlabel, a->zlabel);

		IMPOSE_2ND_SARG(r->attr->title, a->title);
		IMPOSE_2ND_NARG(r->attr->titlefont, a->titlefont);

		IMPOSE_2ND_NARG(r->attr->axes, a->axes);
		INHERIT_2ND_NARG(r->attr->gridstyle, a->gridstyle);
		IMPOSE_2ND_NARG(r->attr->scaling, a->scaling);
		INHERIT_2ND_NARG(r->attr->shading, a->shading);
		INHERIT_2ND_NARG(r->attr->style, a->style);

		INHERIT_2ND_NARG(r->attr->thickness, a->thickness);
		INHERIT_2ND_NARG(r->attr->pointsize, a->pointsize);

		INHERIT_2ND_NARG(r->attr->font, a->font);
		IMPOSE_2ND_NARG(r->attr->axesfont, a->axesfont);
		IMPOSE_2ND_NARG(r->attr->labelfont, a->labelfont);

		IMPOSE_2ND_NARG(r->attr->xticks, a->xticks);
		IMPOSE_2ND_NARG(r->attr->yticks, a->yticks);
		IMPOSE_2ND_NARG(r->attr->zticks, a->zticks);

		a->xmin = MIN(a->xmin, r->attr->xmin);
		a->xmax = MAX(a->xmax, r->attr->xmax);
		a->ymin = MIN(a->ymin, r->attr->ymin);
		a->ymax = MAX(a->ymax, r->attr->ymax);
		a->zmin = MIN(a->zmin, r->attr->zmin);
		a->zmax = MAX(a->zmax, r->attr->zmax);

	}

	if (evalf(a->view_longitude, &d)!=0)
		p->attr->longitude = d;
	if (evalf(a->view_colatitude, &d)!=0)
		p->attr->colatitude = d;

	if (evalf(a->x_viewmin, &d)!=0)
		a->xviewmin = d;
	if (evalf(a->x_viewmax, &d)!=0)
		a->xviewmax = d;
	if (evalf(a->y_viewmin, &d)!=0)
		a->yviewmin = d;
	if (evalf(a->y_viewmax, &d)!=0)
		a->yviewmax = d;
	if (evalf(a->z_viewmin, &d)!=0)
		a->zviewmin = d;
	if (evalf(a->z_viewmax, &d)!=0)
		a->zviewmax = d;

	/* if z-view range is specified, use it */
	if (a->zviewmin!=DBL_MAX) {
		a->zmin = a->zviewmin;
		a->zmax = a->zviewmax;
	}

	/* if x-view and y-view ranges are specified, use them */
	if (a->xviewmin!=DBL_MAX) {
		a->xmin = a->xviewmin;
		a->xmax = a->xviewmax;
		a->ymin = a->yviewmin;
		a->ymax = a->yviewmax;
	}

	saveattr = NULL;
	for (r=p; r!=NULL; r=r->fp) {

		if (r->attr==saveattr)
			continue;

		saveattr = r->attr;

		r->attr->xmin = a->xmin;
		r->attr->xmax = a->xmax;
		r->attr->ymin = a->ymin;
		r->attr->ymax = a->ymax;
		r->attr->zmin = a->zmin;
		r->attr->zmax = a->zmax;
	}

	compute_centroid(p->attr);

	return p;
}
