#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <GL/glx.h>
#include <forms.h>
#include "formdefs.h"
#include "dynagraph.h"

extern char	*prognam;
extern FD_form	*Form;

/* IMPORTANT: the order of entries should match that in the definition
   of "enum PLOTOUT_FORMAT" in dynagraph.h
*/
static char *exts[] = { "jpg", "tiff", "ppm", "ppm", "ps", "ps", "ps", "png" };

static void free_savegraph_struct(SavegraphData *sgd)
{
	testfree(sgd->file);
	free(sgd);
}

void save_graph_to_file(SavegraphData *sgd)
{
	GLuint w, h;
	GLubyte *image;

	if (sgd->fmt == VPS_FORMAT) {

		/* change cursor to a stopwatch */
		if (Form!=NULL) {
			fl_set_cursor(Form->form->window, XC_watch);
			XFlush(fl_display);
		}

		/* determine the file to save it in */
		if (sgd->file==NULL) {
			sgd->file = malloc(strlen(PLOTOUT_FILE_BASENAME)
					+ 1 + strlen(exts[sgd->fmt]) + 1);
			sprintf(sgd->file, "%s.%s", PLOTOUT_FILE_BASENAME,
					exts[sgd->fmt]);
		}

		write_vps(sgd->file);
		free_savegraph_struct(sgd);
		fl_reset_cursor(Form->form->window);
		return;
	}

	if (Form==NULL) {
		printf("\tNo graph to save!\n");
		return;
	}

	assert(Form->glCanvas!=NULL);

	/* determine the plot to save  */
	if (Form->glCanvas->u_vdata==NULL) {
		printf("\tNo graph to save!\n");	
		return;
	}
	
	sgd->plot3d = Form->glCanvas->u_vdata;

	w = Form->glCanvas->w;
	h = Form->glCanvas->h;

	/* Allocate the image buffer */
	if ((image=malloc(3*w*h*sizeof(GLubyte)))==NULL) {
		printf("\twarning: out of memory!\n");
		return;
	}

	/* change cursor to a stopwatch */
	if (Form!=NULL) {
		fl_set_cursor(Form->form->window, XC_watch);
		XFlush(fl_display);
	}

	/* determine the file to save it in */
	if (sgd->file==NULL) {
		sgd->file = malloc(strlen(PLOTOUT_FILE_BASENAME)
				+ 1 + strlen(exts[sgd->fmt]) + 1);
		sprintf(sgd->file, "%s.%s", PLOTOUT_FILE_BASENAME,
				exts[sgd->fmt]);
	}

	glReadPixels(0, 0, w, h, GL_RGB, GL_UNSIGNED_BYTE, image);

	switch (sgd->fmt) {
		case JPEG_FORMAT:
			write_jpeg(sgd->file, image, w, h);
			break;
		case TIFF_FORMAT:
			write_tiff(sgd->file, image, w, h);
			break;
		case PPM6_FORMAT:
			write_ppm6(sgd->file, image, w, h);
			break;
		case PPM3_FORMAT:
			write_ppm3(sgd->file, image, w, h);
			break;
		case PNG_FORMAT:
			write_png(sgd->file, image, w, h);
			break;
		case PS_FORMAT:
			write_ps(sgd->file, image, w, h);
			break;
		case CPS_FORMAT:
			write_cps(sgd->file, image, w, h);
			break;
		default:
			/* should not be here */
			printf("\t%s: internal error in file %s line %d\n",
					prognam, __FILE__, __LINE__);
			break;
	}

	free(image);
	free_savegraph_struct(sgd);
	fl_reset_cursor(Form->form->window);
}
