#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>	/* for isalnum() */
#include <forms.h>
#include "dynagraph.h"
#include "parse.h"
#include "formdefs.h"

#define isalnum2(c)	(isalnum(c)||c=='_')

Expr	*symtab = NULL;
Plot3D	*plottab = NULL;

static short int changed;
extern FD_form *Form;

/* returns symtab entry with a given lval */
Expr *getsym(char *s)
{
	Expr *p;

	for (p=symtab; p!=NULL; p=p->fp)
		if (strcmp(p->lval, s)==0)
			return p;

	return NULL;
}

/* remove symbol(s) of given name from symtab */
void delsym(char *id)
{
	Expr *t, *e;

	while ((e=getsym(id))!=NULL) {
		if (e==symtab)		/* removing table's first entry */
			symtab = symtab->fp;
		else {			/* removing other than first entry */
			for (t=symtab; t->fp!=e; t=t->fp)
				;
			t->fp = e->fp;
		}

		free_expr(e);
	}
}

/* insert sym in symtab */
void putsym(Expr *e)
{
	/* insert symbol in front of symtab */
	e->fp = symtab;
	symtab = e;

	/* if plots with this name exist, delete them */
	if (getplot(e->lval)!=NULL)
		delplot(e->lval);

}

/* retrive plot from plottab */
Plot3D *getplot(char *id)
{
	Plot3D *p;

	for (p=plottab; p!=NULL; p=p->next)
		if (strcmp(p->attr->id, id)==0)
			return p;
	return NULL;
}

/* remove plot(s) of given name from plottab */
void delplot(char *id)
{
	Plot3D *p, *t;

	while ((p=getplot(id))!=NULL) {
		if (p==plottab)		/* removing table's first entry */
			plottab = plottab->next;
		else {			/* removing other than first entry */
			for (t=plottab; t->next!=p; t=t->next)
				;
			t->next = p->next;
		}

		free_plotlist(p);
	}
}

/* After saving a plot, its displaylist indices are no more needed, since
   they will be re-geenrated when the plot is later called up.  Therefore it
   makes sence to call glDeleteLists to free all displaylists associated with
   this plot at this time.  I am not certain whether freeing a displaylist
   frees the memory associated with it.  It yes, then the effort would be
   worthwhile.
*/
void putplot(Plot3D *p)
{
	/* remove exiting plots with this name, if any */
	delplot(p->attr->id);

	/* insert symbol in front of plottab */
	p->next = plottab;
	plottab = p;
	
	/* if a symbols with this name exist, depete them */
	if (getsym(p->attr->id)!=NULL)
		delsym(p->attr->id);


}

/* Routine activated as a result of dynagraph command "id;"
   
   If id represents an expression, show the expression.
   If id represents a plot, display the plot.
   If id is undefined, just echo the id.
*/
void show_sym(char *id)
{
	Expr *e;
	Plot3D *p;
	char *t;

	p = NULL;
	if ((e=getsym(id))!=NULL) {		/* symbol found in symtab */
		t = expr_eval(copy_string(e->lval));
		p = getplot(t);
		if (p==NULL) {			/* not a plot */
			printf("\t%s\n", t);
			free(t);
			return;
		}
		free(t);
	}

	if (p!=NULL || (p=getplot(id))!=NULL) {
		/* don't touch the original, work with a copy */
		p = copy_plotlist(p);
		show_form();
		create_displaylists(p);
		install_plot(p);	/* removes the old */
		/* show obj according to zoom setting */
		zoomdial_cb(Form->zoomdial, 1);
		return;
	}

	printf("\t%s\n", id);			/* symbol not found */
}

/* Returns a pointer to the first occurrance of the "identifier" s in
   string r.  An identifier is a maximal string of alnum2 chars (alnum2
   is defined in dynagraph.h).  */
static char *str_ident(char *r, char *s)
{
	char *p;

	if ((p=strstr(r,s))==NULL)
		return NULL;

	if ( (p==r || !isalnum2((unsigned char)*(p-1)))
			&& !isalnum2((unsigned char)*(p+strlen(s))) )
		return p;
	else
		return str_ident(p+1, s);
}

/* In string r substitutes all occurances of the identifier s
   with t, the latter being enclosed in parentheses, and returns the
   modifiied string.

   Examples:
    args x+y+z, y, Y          result in  x+(Y)+z
    arge (x+y)/(x-y), x, a+b  result in  ((a+b)+y)/((a+b)-y)

   It is assumed that memory for the string r has been allocated
   with malloc().  At each step of the modification we create
   a new string and free() the old one.  If no changes are made, the
   original string is returned.  The global flag "changed" is set
   to 1 if any substitutions take place.
*/
static char *expr_subs(char *r, char *s, char *t)
{
	char *p, *p1, *p2, *rnew;
	int newlen, slen;

	if ((p=str_ident(r,s))==NULL)
		return r;

	slen = strlen(s);
	newlen = strlen(r) + strlen(t) - slen + 2;
	rnew = malloc(newlen+1);

	p1 = r;
	p2 = rnew;

	/* copy the initial part of the string */
	while (p1<p)
		*p2++ = *p1++;

	*p2++ = '(';

	/* copy the replacement string */
	p1 = t;
	while (*p1!='\0')
		*p2++ = *p1++;

	*p2++ = ')';

	p1 = p + slen;

	/* the rest of the string */
	while ((*p2++=*p1++)!='\0')
		;

	free(r);

	changed = 1;

	return expr_subs(rnew, s, t);
}

/* With a sequence of assignments a:=1-b; b:=u+v; the recursive
   evaluation of a results in the string "(1-(u+v))".  The parentheses
   are necessary to preserve the hierarchy of arithmetic operations.
   However with a sequence of assingnments such as a:=b; b:=c; c:=d;
   the recursive evaluation of a results in the string "(((d)))" in
   which the parentheses are redundant.  In this routine we verify if
   the evaluated form of an expression consists of a nested set of
   parentheses enclosing a single identifier, in which case we strip
   the parentheses and return the identifier.  Note that we don't want
   to strip the outer parentheses of an expression like "(1-r)/(1+r)".
*/
static char *strip_parens(char *s)
{
	char *p, *p1, *p2;

	p = s;
	while (*p=='(')
		p++;

	p1 = p;

	while (isalnum2((unsigned char)*p))
		p++;

	p2 = p;

	while (*p==')')
		p++;

	if (*p!='\0')
		return s;


	/* string is of the form (((...ident...))).  p1 and p2 mark
	   the beginning and the end of the ident. */

	*p2 = '\0';

	p = malloc(p2-p1+1);
	strcpy(p, p1);
	free(s);
	return p;
}

/* In a given string, which represents a mathematical symbolic expression,
   recursively substitutes symbols defined in symtab and returns an
   equivalent expression involving primitive (undefined) symbols only.
*/
char *expr_eval(char *s)
{
	Expr *t;
	int i = RLIMIT;

	do {
		changed = 0;
		for (t=symtab; t!=NULL; t=t->fp)
			s = expr_subs(s, t->lval, t->rval);
		if (!changed)
			return strip_parens(s);
	} while (i-->0);

	printf("\terror: recursive definition of symbol\n");
	return NULL;
}

void assign_sym(Expr *e)
{
	Plot3D *p;
	Expr *t;
	char *s;

	if (strcmp(e->lval, e->rval)==0) {	/* x := x; */
		if ((t=getsym(e->rval))!=NULL)
			delsym(e->lval);
		else if ((p=getplot(e->rval))!=NULL)
			delplot(e->lval);
		else
#ifdef USE
			printf("\twarning: assignment %s := %s has no effect!\n", e->lval,e->rval);
#else
			; /* empty */
#endif /* USE */
		free_expr(e);

		return;
	}

	putsym(e);

	/* test if this assignment causes infinite recursion */
	s = expr_eval(copy_string(e->rval));

	if (s==NULL)
		delsym(e->lval);
	else
		free(s);
}

void re_start(void)
{
	while (symtab!=NULL)
		delsym(symtab->lval);

	while (plottab!=NULL)
		delplot(plottab->attr->id);
}

void dump_symtab(void)
{
	Plot3D *p;
	Expr *e;
	char *t;

	printf("Symbol table dump:\n");
	for (e=symtab; e!=NULL; e=e->fp) {
		t = expr_eval(copy_string(e->lval));
		printf("\t%s\t:: %s\n\t\t-> %s\n", e->lval, e->rval, t);
		if (getplot(t)!=NULL)
			printf("\t\t-> plot3d(...)\n");
		testfree(t);
	}

	printf("Plot table dump:\n");
	for (p=plottab; p!=NULL; p=p->next)
		printf("\t%s = plot3d(...)\n", p->attr->id);
}
