/* write_ppm[3]() modeled after the routine in osmesa.c distributed with Mesa.
   Accroding to a note in that file:
 
   PPM output (PPM6) provided by Joerg Schmalzl
   ASCII PPM (PPM3) output added by Brian Paul
*/

#include <stdio.h>
#include "dynagraph.h"	/* for the DYNAGRAPH_VERSION string */

extern char *prognam;

/* save file in binary ppm format */
void write_ppm6(char *file, unsigned char *image, unsigned int w, unsigned int h)
{
	FILE *fp;
	unsigned char *p;
	unsigned int x, y;
	int i;

	if ((fp=fopen(file, "w"))==NULL) {
		printf("\terror: unable to open file '%s' for writing\n", file);
		return;
	}

	/* ready to write PPM file */
	p = image;
	fprintf(fp, "P6\n");
	fprintf(fp, "# PPM file created by dynagraph %s\n", DYNAGRAPH_VERSION);
	fprintf(fp, "%i %i\n", w, h);
	fprintf(fp, "255\n");
	fclose(fp);
	/* reopen file in binary append mode */
	fp = fopen(file, "ab");
	for (y=h; y>0;  ) {
		y--;
		for (x=0; x<w; x++) {
			i = (y*w + x) * 3;
			fputc(p[i], fp);	/* write red */
			fputc(p[i+1], fp);	/* write green */
			fputc(p[i+2], fp);	/* write blue */
		}
	}

	fclose(fp);
}

/* Save file in ASCII ppm format
   Note that the ppm standard states that no line should be longer
   than 70 characters in an ascii ppm file.
*/
void write_ppm3(char *file, unsigned char *image, unsigned int w, unsigned int h)
{
	FILE *fp;
	unsigned char *p;
	unsigned int x, y;
	int i, counter;

	if ((fp=fopen(file, "w"))==NULL) {
		printf("\terror: unable to open file '%s' for writing\n", file);
		return;
	}

	/* ready to write PPM file */
	p = image;
	fprintf(fp, "P3\n");
	fprintf(fp, "# PPM file created by dynagraph %s\n", DYNAGRAPH_VERSION);
	fprintf(fp, "%i %i\n", w, h);
	fprintf(fp, "255\n");
	for (y=h; y>0; ) {
		y--;
		counter=0;
		for (x=0; x<w; x++) {
			i = (y*w + x) * 3;
			fprintf(fp, " %3d %3d %3d", p[i], p[i+1], p[i+2]);
			counter++;
			if (counter%5 == 0)	/* 60 chars per line */
				fprintf(fp, "\n");
		}
		if (counter%5 != 0)
			fprintf(fp, "\n");
	}

	fclose(fp);
}
